@@ -8,6 +8,7 @@
 #include "../src/builtin/ne2000.h"
 #include "../src/builtin/glide2x.h"
 #include "../src/builtin/emsmagic.h"
+#include "../src/builtin/shutdown.h"
 #include "../src/builtin/textutil.h"
 #include "../src/builtin/4DOS_img.h"
 
@@ -20,20 +21,24 @@ extern struct BuiltinFileBlob bfb_DOS4GW_EXE;		// DOS4GW.EXE
 extern struct BuiltinFileBlob bfb_DOSIDLE_EXE;		// DOSIDLE.EXE
 extern struct BuiltinFileBlob bfb_HEXMEM16_EXE;		// HEXMEM16.EXE
 extern struct BuiltinFileBlob bfb_HEXMEM32_EXE;		// HEXMEM32.EXE
-extern struct BuiltinFileBlob bfb_REPLACE_EXE;		// REPLACE.EXE
+extern struct BuiltinFileBlob bfb_REPLACE_EXE;		// REPLACE.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_SORT_EXE;		// SORT.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_MOVE_EXE;		// MOVE.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_MEM_EXE;		// MEM.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_FIND_EXE;		// FIND.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_DEBUG_EXE;		// DEBUG.EXE (FreeDOS)
+extern struct BuiltinFileBlob bfb_PRINT_COM;		// PRINT.COM (FreeDOS)
 extern struct BuiltinFileBlob bfb_XCOPY_EXE;		// XCOPY.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_APPEND_EXE;		// APPEND.EXE (FreeDOS)
-extern struct BuiltinFileBlob bfb_EDIT_COM;		// EDIT.COM (FreeDOS)
+extern struct BuiltinFileBlob bfb_EDLIN_EXE;		// EDLIN.EXE (FreeDOS)
+extern struct BuiltinFileBlob bfb_EDIT_COM;		// EDIT.COM
 extern struct BuiltinFileBlob bfb_DEVICE_COM;		// DEVICE.COM
 extern struct BuiltinFileBlob bfb_BUFFERS_COM;		// BUFFERS.COM
 extern struct BuiltinFileBlob bfb_LASTDRIV_COM;		// LASTDRIV.COM
+extern struct BuiltinFileBlob bfb_FILES_COM;		// FILES.COM
 extern struct BuiltinFileBlob bfb_FCBS_COM;		// FCBS.COM
-extern struct BuiltinFileBlob bfb_COPY_EXE;		// COPY.EXE
+extern struct BuiltinFileBlob bfb_FC_EXE;		// FC.EXE (FreeDOS)
+extern struct BuiltinFileBlob bfb_COMP_COM;		// COMP.COM (FreeDOS)
 extern struct BuiltinFileBlob bfb_EVAL_EXE;		// EVAL.EXE
 extern struct BuiltinFileBlob bfb_EVAL_HLP;		// EVAL.HLP
 extern struct BuiltinFileBlob bfb_28_COM;		// 28.COM
@@ -60,6 +65,8 @@ extern struct BuiltinFileBlob bfb_EGA_COM;	// EGA.COM
 extern struct BuiltinFileBlob bfb_CLR_COM;	// CLR.COM
 extern struct BuiltinFileBlob bfb_CGA_COM;	// CGA.COM
 extern struct BuiltinFileBlob bfb_EMSMAGIC_COM;	// EMSMAGIC.COM
+extern struct BuiltinFileBlob bfb_SHUTDOWN_COM;	// SHUTDOWN.COM
+extern struct BuiltinFileBlob bfb_DISKCOPY_EXE;	// DISKCOPY.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_DEFRAG_EXE;	// DEFRAG.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_FDISK_EXE;	// FDISK.EXE (FreeDOS)
 extern struct BuiltinFileBlob bfb_FORMAT_EXE;	// FORMAT.EXE (FreeDOS)
