@@ -38,13 +38,16 @@ struct CommandTail{
 #pragma pack ()
 #endif
 
-#define IS_DOS_JAPANESE (mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0x81 && mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0x9F)
-#define IS_DOS_CJK ((mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0x81 || mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0xA1) && (mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0x9F || mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0xFE))
+extern bool dos_kernel_disabled;
+
+#define IS_DOS_JAPANESE (!dos_kernel_disabled && mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0x81 && mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0x9F)
+#define IS_DOS_CJK (!dos_kernel_disabled && ((mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0x81 || mem_readb(Real2Phys(dos.tables.dbcs) + 0x02) == 0xA1) && (mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0x9F || mem_readb(Real2Phys(dos.tables.dbcs) + 0x03) == 0xFE)))
 #define IS_DOSV (dos.set_jdosv_enabled || dos.set_kdosv_enabled || dos.set_cdosv_enabled || dos.set_pdosv_enabled)
 #define IS_JDOSV (dos.set_jdosv_enabled)
 #define IS_KDOSV (dos.set_kdosv_enabled)
 #define IS_CDOSV (dos.set_cdosv_enabled)
 #define IS_PDOSV (dos.set_pdosv_enabled)
+#define IS_J3100 (dos.set_j3100_enabled)
 
 #define	EXT_DEVICE_BIT				0x0200
 
@@ -388,6 +391,7 @@ public:
 	void	MakeNew				(uint16_t mem_size);
 	void	CopyFileTable		(DOS_PSP* srcpsp,bool createchildpsp);
 	uint16_t	FindFreeFileEntry	(void);
+	void	CloseFile			(const char *name);
 	void	CloseFiles			(void);
 
 	void	SaveVectors			(void);
@@ -767,6 +771,7 @@ struct DOS_Block {
     bool set_jdosv_enabled = false;
     bool set_kdosv_enabled = false;
     bool set_pdosv_enabled = false;
+    bool set_j3100_enabled = false;
     bool im_enable_flag;
     uint16_t dcp;	// Device command packet
 };
