@@ -257,6 +257,18 @@
 			reg_eax=(uint32_t)(tsc&0xffffffff);
 		}
 		break;
+	CASE_0F_B(0x34)												/* SYSENTER */
+		{
+			if (CPU_ArchitectureType<CPU_ARCHTYPE_PENTIUMII) goto illegal_opcode;
+			if (!CPU_SYSENTER()) goto illegal_opcode;
+		}
+		continue;
+	CASE_0F_B(0x35)												/* SYSEXIT */
+		{
+			if (CPU_ArchitectureType<CPU_ARCHTYPE_PENTIUMII) goto illegal_opcode;
+			if (!CPU_SYSEXIT()) goto illegal_opcode;
+		}
+		continue;
 
 	// Pentium Pro Conditional Moves
 	CASE_0F_W(0x40)												/* CMOVO */
