@@ -48,18 +48,23 @@
 	CASE_W(0x0e)												/* PUSH CS */		
 		Push_16(SegValue(cs));break;
 	CASE_B(0x0f)												/* 2 byte opcodes*/
-#if CPU_CORE < CPU_ARCHTYPE_286
-		if (CPU_ArchitectureType < CPU_ARCHTYPE_286) {
-			/* 8086 emulation: treat as "POP CS" */
-			if (CPU_PopSeg(cs,false)) RUNEXCEPTION();
-			break;
-		}
-		else
+#if CPU_CORE == CPU_ARCHTYPE_8086
+	/* 8086 emulation: treat as "POP CS" */
+	SAVEIP;
+	if (CPU_PopSeg(cs,false)) RUNEXCEPTION();
+	goto skip_saveip;
+#else
+#  if CPU_CORE == CPU_ARCHTYPE_286
+	/* try not to slow down 386+ emulation with a check for 186 every time we hit an opcode starting with 0x0F,
+	 * do this check only for the 80186 and 286 emulation provided by normal/prefetch 286 core code. */
+        if (CPU_ArchitectureType < CPU_ARCHTYPE_286)
+            goto illegal_opcode;
+#  endif
+
+        core.opcode_index|=OPCODE_0F;
+        goto restart_opcode;
+        break;
 #endif
-		{
-			core.opcode_index|=OPCODE_0F;
-			goto restart_opcode;
-		} break;
 	CASE_B(0x10)												/* ADC Eb,Gb */
 		RMEbGb(ADCB);break;
 	CASE_W(0x11)												/* ADC Ew,Gw */
@@ -228,7 +233,9 @@
 	CASE_W(0x5f)												/* POP DI */
 		reg_di=Pop_16();break;
 	CASE_W(0x60)												/* PUSHA */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_O); break; /* alias of 0x70, jo. see also [http://www.os2museum.com/wp/undocumented-8086-opcodes/] */
+#else
 		{
 			uint32_t old_esp = reg_esp;
 			try {
@@ -243,8 +250,11 @@
 				throw;
 			}
 		} break;
+#endif
 	CASE_W(0x61)												/* POPA */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NO); break; /* alias of 0x71, jno */
+#else
 		{
 			uint32_t old_esp = reg_esp;
 			try {
@@ -258,8 +268,11 @@
 				throw;
 			}
 		} break;
+#endif
 	CASE_W(0x62)												/* BOUND */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_B); break; /* alias of 0x72, jb */
+#else
 		{
 			int16_t bound_min, bound_max;
 			GetRMrw;GetEAa;
@@ -270,8 +283,16 @@
 			}
 		}
 		break;
+#endif
 	CASE_W(0x63)												/* ARPL Ew,Rw */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_286) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NB); break; /* alias of 0x73, jnb */
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_286
+        if (CPU_ArchitectureType < CPU_ARCHTYPE_286) goto illegal_opcode;
+        else
+#endif
+#if CPU_CORE >= CPU_ARCHTYPE_286
 		{
 			if ((reg_flags & FLAG_VM) || (!cpu.pmode)) goto illegal_opcode;
 			GetRMrw;
@@ -286,52 +307,104 @@
 			}
 		}
 		break;
+#endif
 	CASE_B(0x64)												/* SEG FS: */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_386) goto illegal_opcode;
-		DO_PREFIX_SEG(fs);break;
-	CASE_B(0x65)												/* SEG GS: */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_386) goto illegal_opcode;
-		DO_PREFIX_SEG(gs);break;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_Z); break; /* alias of 0x74, jz */
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_286
+        goto illegal_opcode;
+#endif
 #if CPU_CORE >= CPU_ARCHTYPE_386
+		DO_PREFIX_SEG(fs);break;
+#endif
+	CASE_B(0x65)												/* SEG GS: */
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NZ); break; /* alias of 0x75, jnz */
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_286
+        goto illegal_opcode;
+#endif
+#if CPU_CORE >= CPU_ARCHTYPE_386
+		DO_PREFIX_SEG(gs);break;
+#endif
+
 	CASE_B(0x66)												/* Operand Size Prefix (386+) */
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_BE); break; /* alias of 0x76, jbe */
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_286
+        goto illegal_opcode;
+#endif
+#if CPU_CORE >= CPU_ARCHTYPE_386
 		core.opcode_index=(cpu.code.big^0x1u)*0x200u;
 		goto restart_opcode;
 #endif
-#if CPU_CORE >= CPU_ARCHTYPE_386
 	CASE_B(0x67)												/* Address Size Prefix (386+) */
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NBE); break; /* alias of 0x77, jnbe */
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_286
+        goto illegal_opcode;
+#endif
+#if CPU_CORE >= CPU_ARCHTYPE_386
 		DO_PREFIX_ADDR();
 #endif
 	CASE_W(0x68)												/* PUSH Iw */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_S); break; /* alias of 0x78, js */
+#else
 		Push_16(Fetchw());break;
+#endif
 	CASE_W(0x69)												/* IMUL Gw,Ew,Iw */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NS); break; /* alias of 0x79, jns */
+#else
 		RMGwEwOp3(DIMULW,Fetchws());
 		break;
+#endif
 	CASE_W(0x6a)												/* PUSH Ib */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_P); break; /* alias of 0x7A, jp */
+#else
 		Push_16((uint16_t)Fetchbs());
 		break;
+#endif
 	CASE_W(0x6b)												/* IMUL Gw,Ew,Ib */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NP); break; /* alias of 0x7B, jnp */
+#else
 		RMGwEwOp3(DIMULW,Fetchbs());
 		break;
+#endif
 	CASE_B(0x6c)												/* INSB */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_L); break; /* alias of 0x7C, jl */
+#else
 		if (CPU_IO_Exception(reg_dx,1)) RUNEXCEPTION();
 		DoString(R_INSB);break;
+#endif
 	CASE_W(0x6d)												/* INSW */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NL); break; /* alias of 0x7D, jnl */
+#else
 		if (CPU_IO_Exception(reg_dx,2)) RUNEXCEPTION();
 		DoString(R_INSW);break;
+#endif
 	CASE_B(0x6e)												/* OUTSB */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_LE); break; /* alias of 0x7E, jle */
+#else
 		if (CPU_IO_Exception(reg_dx,1)) RUNEXCEPTION();
 		DoString(R_OUTSB);break;
+#endif
 	CASE_W(0x6f)												/* OUTSW */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+        JumpCond16_b(TFLG_NLE); break; /* alias of 0x7F, jnle */
+#else
 		if (CPU_IO_Exception(reg_dx,2)) RUNEXCEPTION();
 		DoString(R_OUTSW);break;
+#endif
 	CASE_W(0x70)												/* JO */
 		JumpCond16_b(TFLG_O);break;
 	CASE_W(0x71)												/* JNO */
@@ -558,17 +631,96 @@
 			if (rm >= 0xc0 ) {GetEArw;val=*earw;}
 			else {GetEAa;val=LoadMw(eaa);}
 			switch (which) {
-#if CPU_CORE <= CPU_ARCHTYPE_8086
-			case 0x01:					/* MOV CS,Ew (8086 only) */
+            case 0x00:                  /* MOV ES,Ew */
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+            case 0x01:                  /* MOV CS,Ew (8086) */
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+#if CPU_CORE == CPU_ARCHTYPE_286
+            case 0x01:                  /* MOV CS,Ew (186) */
+                if(CPU_ArchitectureType == CPU_ARCHTYPE_286) goto illegal_opcode;
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#endif
+#endif
+            case 0x02:                  /* MOV SS,Ew */
+                CPU_Cycles++; //Always do another instruction
+            case 0x03:                  /* MOV DS,Ew */
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#if CPU_CORE == CPU_ARCHTYPE_8086
+            case 0x04:                  /* Alias of MOV ES,Ew (8086) */
+                which = 0;
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+#if CPU_CORE == CPU_ARCHTYPE_286
+            case 0x04:                  /* Alias of MOV ES,Ew (186) */
+                if(CPU_ArchitectureType == CPU_ARCHTYPE_286) goto illegal_opcode;
+                which = 0;
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+            case 0x04:                  /* MOV FS,Ew (386+) */
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#endif
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_8086
+            case 0x05:                  /* Alias of MOV CS,Ew (8086) */
+                which = 1;
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+#if CPU_CORE == CPU_ARCHTYPE_286
+            case 0x05:                  /* Alias of MOV CS,Ew (186) */
+                if(CPU_ArchitectureType == CPU_ARCHTYPE_286) goto illegal_opcode;
+                which = 1;
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+            case 0x05:                  /* MOV FS,Ew (386+) */
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#endif
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_8086
+            case 0x06:                  /* Alias of MOV SS,Ew (8086) */
+                which = 2;
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+#if CPU_CORE == CPU_ARCHTYPE_286
+            case 0x06:                  /* Alias of MOV SS,Ew (186) */
+                if(CPU_ArchitectureType == CPU_ARCHTYPE_286) goto illegal_opcode;
+                which = 2;
+                CPU_Cycles++; //Always do another instruction
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#endif
+#endif
+#if CPU_CORE == CPU_ARCHTYPE_8086
+            case 0x07:                  /* Alias of MOV DS,Ew (8086) */
+                which = 3;
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#else
+#if CPU_CORE == CPU_ARCHTYPE_286
+            case 0x07:                  /* Alias of MOV DS,Ew (186) */
+                if(CPU_ArchitectureType == CPU_ARCHTYPE_286) goto illegal_opcode;
+                which = 3;
+                if(CPU_SetSegGeneral((SegNames)which, val)) RUNEXCEPTION();
+                break;
+#endif
 #endif
-			case 0x02:					/* MOV SS,Ew */
-				CPU_Cycles++; //Always do another instruction
-			case 0x00:					/* MOV ES,Ew */
-			case 0x03:					/* MOV DS,Ew */
-			case 0x05:					/* MOV GS,Ew */
-			case 0x04:					/* MOV FS,Ew */
-				if (CPU_SetSegGeneral((SegNames)which,val)) RUNEXCEPTION();
-				break;
 			default:
 				goto illegal_opcode;
 			}
@@ -741,14 +893,19 @@
 		reg_di=Fetchw();break;
 #if CPU_CORE >= CPU_ARCHTYPE_80186
 	CASE_B(0xc0)												/* GRP2 Eb,Ib */
-		if (CPU_ArchitectureType < CPU_ARCHTYPE_80186) abort();
 		GRP2B(Fetchb());break;
 	CASE_W(0xc1)												/* GRP2 Ew,Ib */
-		if (CPU_ArchitectureType < CPU_ARCHTYPE_80186) abort();
 		GRP2W(Fetchb());break;
+#else
+    CASE_W(0xc0)												/* Alias of RETN Iw (0xC2) on 8086 */
+        goto opcode_c2;
+    CASE_W(0xc1)												/* Alias of RETN (0xC3) on 8086 */
+        reg_eip = Pop_16();
+        continue;
 #endif
 	CASE_W(0xc2)												/* RETN Iw */
 		{
+        opcode_c2:
 			uint32_t old_esp = reg_esp;
 
 			try {
@@ -798,8 +955,8 @@
 			else {GetEAa;SaveMw(eaa,Fetchw());}
 			break;
 		}
+#if CPU_CORE >= CPU_ARCHTYPE_80186
 	CASE_W(0xc8)												/* ENTER Iw,Ib */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
 		{
 			Bitu bytes=Fetchw();
 			Bitu level=Fetchb();
@@ -807,7 +964,6 @@
 		}
 		break;
 	CASE_W(0xc9)												/* LEAVE */
-		if (CPU_ArchitectureType<CPU_ARCHTYPE_80186) goto illegal_opcode;
 		{
 			uint32_t old_esp = reg_esp;
 
@@ -822,14 +978,22 @@
 				throw;
 			}
 		} break;
+#else
+    CASE_W(0xc8)												/* Alias of RETF Iw (0xCA) on 8086 */
+        goto opcode_ca;
+    CASE_W(0xc9)												/* Alias of RETF (0xCB) on 8086 */
+        goto opcode_cb;
+#endif
 	CASE_W(0xca)												/* RETF Iw */
 		{
+        opcode_ca:
 			Bitu words=Fetchw();
 			FillFlags();
 			CPU_RET(false,words,GETIP);
 			continue;
 		}
-	CASE_W(0xcb)												/* RETF */			
+	CASE_W(0xcb)												/* RETF */
+        opcode_cb:
 		FillFlags();
 		CPU_RET(false,0,GETIP);
 		continue;
@@ -1102,15 +1266,21 @@
 		IO_WriteW(reg_dx,reg_ax);
 		break;
 	CASE_B(0xf0)												/* LOCK */
+        opcode_f0:
 // todo: make an option to show this
 //		LOG(LOG_CPU,LOG_NORMAL)("CPU:LOCK"); /* FIXME: see case D_LOCK in core_full/load.h */
 		break;
+#if CPU_CORE >= CPU_ARCHTYPE_80186
 	CASE_B(0xf1)												/* ICEBP */
 		CPU_SW_Interrupt_NoIOPLCheck(1,GETIP);
 #if CPU_TRAP_CHECK
 		cpu.trap_skip=true;
 #endif
 		continue;
+#else
+     CASE_B(0xf1)                                                /* Believed to be alias of LOCK (0xf0) on 8086 */
+        goto opcode_f0;
+#endif
 	CASE_B(0xf2)												/* REPNZ */
 		DO_PREFIX_REP(false);	
 		break;		
