@@ -22,6 +22,10 @@
 #include "pic.h"
 #include "fpu.h"
 
+/* Do not emulate segment limit exceptions on 8086. This turns the if() statements into if (0)
+ * and the C++ compiler optimizer should then completely omit the code for the 8086 cputype. */
+#define do_seg_limits (0)
+
 /* 8086: Apparently exceptions like DIVIDE BY ZERO are given a return address of the next instruction */
 #define PRE_EXCEPTION SAVEIP
 
@@ -124,7 +128,12 @@ extern Bitu cycle_count;
 #define TEST_PREFIX_REP		(core.prefixes & PREFIX_REP)
 
 #define DO_PREFIX_SEG(_SEG)					\
-    if (GETFLAG(IF) && CPU_Cycles <= 0 && !mustCompleteInstruction) goto prefix_out; \
+    if (GETFLAG(IF) && CPU_Cycles <= 0 && !mustCompleteInstruction) \
+    {   \
+        uint8_t next = LoadMb(core.cseip + 1);  \
+        if (next == 0xf2 || next == 0xf3)   \
+            goto prefix_out;    \
+    }   \
 	BaseDS=SegBase(_SEG);					\
 	BaseSS=SegBase(_SEG);					\
 	core.base_val_ds=_SEG;					\
@@ -252,6 +261,7 @@ restart_opcode:
 		}
 		SAVEIP;
 	}
+skip_saveip:
 	FillFlags();
 	return CBRET_NONE;
 /* 8086/286 multiple prefix interrupt bug emulation.
