@@ -227,6 +227,10 @@ uint32_t DOS_CheckExtDevice(const char *name, bool already_flag) {
 						}
 					}
 				}
+				// Exclude the default CON and NUL
+				if(real_readd(seg, off + 6) == 0 || real_readd(seg, off + 6) == 0xffffffff) {
+					return 0;
+				}
 				return (uint32_t)seg << 16 | (uint32_t)off;
 			}
 		}
