@@ -39,10 +39,11 @@ public:
 	static void ChangeDisk(int drive, DOS_Drive* disk);
 	static void InitializeDrive(int drive);
 	static int UnmountDrive(int drive);
+	static int GetDisksSize(int drive);
 	static char * GetDrivePosition(int drive);
 //	static void CycleDrive(bool pressed);
 //	static void CycleDisk(bool pressed);
-	static void CycleDisks(int drive, bool notify);
+	static void CycleDisks(int drive, bool notify, int position=0);
 	static void CycleAllDisks(void);
 	static void CycleAllCDs(void);
 	static void Init(Section* s);
@@ -660,12 +661,13 @@ public:
 	virtual bool isRemote(void);
 	virtual bool isRemovable(void);
 	virtual Bits UnMount(void);
+	bool loadImage();
 	bool readSector(uint8_t *buffer, uint32_t sector);
+	void setFileName(const char* fileName);
 	virtual char const* GetLabel(void) {return discLabel;};
 	virtual void Activate(void);
 private:
     int  readDirEntry(isoDirEntry* de, const uint8_t* data);
-	bool loadImage();
 	bool lookupSingle(isoDirEntry *de, const char *name, uint32_t sectorStart, uint32_t length);
 	bool lookup(isoDirEntry *de, const char *path);
 	int  UpdateMscdex(char driveLetter, const char* path, uint8_t& subUnit);
