@@ -240,6 +240,9 @@ static const char *def_menu_cpu_speed[] =
     "cpu586-120",
     "cpu586-133",
     "cpu586-166",
+    "cpuak6-166",
+    "cpuak6-200",
+    "cpuak6-300",
     NULL
 };
 
@@ -279,6 +282,7 @@ static const char *def_menu_cpu_type[] =
     "cputype_pentium",
     "cputype_pentium_mmx",
     "cputype_ppro_slow",
+    "cputype_pentium_ii",
     "--",
     "cputype_experimental",
     NULL
@@ -401,6 +405,7 @@ static const char *def_menu_video_ttf[] =
     "ttf_dbcs_sbcs",
     "ttf_autoboxdraw",
     "ttf_halfwidthkana",
+    "ttf_extcharset",
     NULL
 };
 #endif
@@ -499,7 +504,7 @@ static const char *def_menu_video[] =
 #endif
     "VideoPC98Menu",
     "Video3dfxMenu",
-#if defined(C_D3DSHADERS) || defined(C_OPENGL)
+#if defined(C_D3DSHADERS) || defined(C_OPENGL) || defined(USE_TTF)
     "--",
 #endif
 #ifdef C_D3DSHADERS
@@ -1213,7 +1218,7 @@ LPWSTR getWString(std::string str, wchar_t *def, wchar_t*& buffer) {
         cp = msgcodepage;
     else if ((!dos.loaded_codepage || dos_kernel_disabled || force_conversion) && section!=NULL && !control->opt_noconfig) {
         char *countrystr = (char *)section->Get_string("country"), *r=strchr(countrystr, ',');
-        if (r!=NULL && *(r+1)) {
+        if (r!=NULL && *(r+1) && !IS_PC98_ARCH && !IS_JEGA_ARCH && !IS_DOSV) {
             cp = atoi(trim(r+1));
             if ((cp<1 || !isSupportedCP(cp)) && msgcodepage>0) cp = msgcodepage;
         } else if (msgcodepage>0)
