@@ -44,6 +44,8 @@
 #define OPL3_INTERNAL_FREQ    14400000  // The OPL3 operates at 14.4MHz
 
 bool adlib_force_timer_overflow_on_polling = false;
+bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
+extern std::string pathopl;
 
 namespace OPL2 {
 	#include "opl.cpp"
@@ -1147,10 +1149,12 @@ void OPL_SaveRawEvent(bool pressed) {
 		delete module->capture;
 		module->capture = 0;
 		LOG_MSG("Stopped Raw OPL capturing.");
+		if (pathopl.size()) systemmessagebox("Recording completed",("Saved Raw OPL output to the file:\n\n"+pathopl).c_str(),"ok", "info", 1);
 	} else {
 		LOG_MSG("Preparing to capture Raw OPL, will start with first note played.");
 		module->capture = new Adlib::Capture( &module->cache );
 	}
+	pathopl = "";
 
 	mainMenu.get_item("mapper_caprawopl").check(module->capture != NULL).refresh_item(mainMenu);
 }
@@ -1203,10 +1207,7 @@ Module::Module( Section* configuration ) : Module_base(configuration) {
 	//Used to be 2.0, which was measured to be too high. Exact value depends on card/clone.
 	mixerChan->SetScale( 1.5f );  
 
-	if (oplemu == "fast") {
-		handler = new DBOPL::Handler();
-	}
-	else if (oplemu == "compat") {
+	if (oplemu == "compat") {
 		if (oplmode == OPL_opl2) {
 			handler = new OPL2::Handler();
 		}
@@ -1235,8 +1236,11 @@ Module::Module( Section* configuration ) : Module_base(configuration) {
 		else {
 			handler = new MAMEOPL3::Handler();
 		}
-	} else {
-		handler = new DBOPL::Handler();
+	} 
+	//Fall back to dbop, will also catch auto
+	else if (oplemu == "fast" || 1) {
+		const bool opl3Mode = oplmode >= OPL_opl3;
+		handler = new DBOPL::Handler( opl3Mode );
 	}
 	usedoplemu = oplemu;
 	handler->Init( rate );
