@@ -1,14 +1,9 @@
 #include "config.h"
 
-#if C_NE2000
-
-#if defined(WIN32)
-  #define HAVE_REMOTE
-#endif
-
 #include "dosbox.h"
 #include <string.h>
 #include <stdio.h>
+#include "logging.h"
 #include "support.h"
 #include "inout.h"
 #include "setup.h"
@@ -53,28 +48,11 @@
 
 #include "ne2000.h"
 
-#include "pcap.h"
-// Handle to WinPCap device
-pcap_t *adhandle = 0;
+#include "ethernet.h"
+EthernetConnection* ethernet = nullptr;
+
 static void NE2000_TX_Event(Bitu val);
-
-#ifdef WIN32
-// DLL loading
-#define pcap_sendpacket(A,B,C)			PacketSendPacket(A,B,C)
-#define pcap_close(A)					PacketClose(A)
-#define pcap_freealldevs(A)				PacketFreealldevs(A)
-#define pcap_open(A,B,C,D,E,F)			PacketOpen(A,B,C,D,E,F)
-#define pcap_next_ex(A,B,C)				PacketNextEx(A,B,C)
-#define pcap_findalldevs_ex(A,B,C,D)	PacketFindALlDevsEx(A,B,C,D)
-
-int (*PacketSendPacket)(pcap_t *, const u_char *, int) = 0;
-void (*PacketClose)(pcap_t *) = 0;
-void (*PacketFreealldevs)(pcap_if_t *) = 0;
-pcap_t* (*PacketOpen)(char const *,int,int,int,struct pcap_rmtauth *,char *) = 0;
-int (*PacketNextEx)(pcap_t *, struct pcap_pkthdr **, const u_char **) = 0;
-int (*PacketFindALlDevsEx)(char *, struct pcap_rmtauth *, pcap_if_t **, char *) = 0;
-
-#endif
+void VFILE_Remove(const char *name,const char *dir = "");
 
 //Never completely fill the ne2k ring so that we never
 // hit the unclear completely full buffer condition.
@@ -89,6 +67,7 @@ int (*PacketFindALlDevsEx)(char *, struct pcap_rmtauth *, pcap_if_t **, char *)
 #define BX_RESET_HARDWARE 0
 #define BX_RESET_SOFTWARE 1
 
+bool addne2k = false;
 static char bxtmp[1024];
 
 static inline void BX_INFO(const char *msg,...) {
@@ -301,7 +280,7 @@ bx_ne2k_c::write_cr(uint32_t value)
     // Send the packet to the system driver
 	/* TODO: Transmit packet */
     //BX_NE2K_THIS ethdev->sendpkt(& BX_NE2K_THIS s.mem[BX_NE2K_THIS s.tx_page_start*256 - BX_NE2K_MEMSTART], BX_NE2K_THIS s.tx_bytes);
-	pcap_sendpacket(adhandle,&s.mem[s.tx_page_start*256 - BX_NE2K_MEMSTART], s.tx_bytes);
+    	ethernet->SendPacket(&s.mem[s.tx_page_start*256 - BX_NE2K_MEMSTART], s.tx_bytes);
 	// some more debug
 	if (BX_NE2K_THIS s.tx_timer_active) {
       BX_PANIC(("CR write, tx timer still active"));
@@ -1468,23 +1447,17 @@ static void NE2000_TX_Event(Bitu val) {
 }
 
 static void NE2000_Poller(void) {
-	int res;
-	struct pcap_pkthdr *header;
-	u_char *pkt_data;
-//#if 0
-	while((res = pcap_next_ex( adhandle, &header, (const u_char **)&pkt_data)) > 0) {
+	ethernet->GetPackets([](const uint8_t* packet, int len) {
 		//LOG_MSG("NE2000: Received %d bytes", header->len);
-		
+
 		// don't receive in loopback modes
 		if((theNE2kDevice->s.DCR.loop == 0) || (theNE2kDevice->s.TCR.loop_cntl != 0))
 			return;
-		theNE2kDevice->rx_frame(pkt_data, header->len);
-	}
-//#endif
+		theNE2kDevice->rx_frame(packet, len);
+	});
 }
-#ifdef WIN32
-#include <windows.h>
-#endif
+
+extern std::string niclist;
 
 class NE2K: public Module_base {
 private:
@@ -1503,61 +1476,20 @@ public:
 		// enabled?
 
 		if(!section->Get_bool("ne2000")) {
+			addne2k = false;
 			load_success = false;
 			return;
 		}
 
-#ifdef WIN32
-/*		
-		int (*PacketSendPacket)(pcap_t *, const u_char *, int);
-		void (*PacketClose)(pcap_t *);
-		void (*PacketFreealldevs)(pcap_if_t *);
-		pcap_t* (*PacketOpen)(char const *,int,int,int,struct pcap_rmtauth *,char *);
-		int (*PacketNextEx)(pcap_t *, struct pcap_pkthdr **, const u_char **);
-		int (*PacketFindALlDevsEx)(char *, struct pcap_rmtauth *, pcap_if_t **, char *);
-*/
-		// init the library
-		HINSTANCE pcapinst;
-		pcapinst = LoadLibrary("WPCAP.DLL");
-		if(pcapinst==NULL) {
-			LOG_MSG("WinPcap has to be installed for the NE2000 to work.");
+		const char* backendstring = section->Get_string("backend");
+		ethernet = OpenEthernetConnection(backendstring);
+		if(!ethernet)
+		{
+			LOG_MSG("NE2000: Failed to open Ethernet backend %s", backendstring);
+			addne2k = false;
 			load_success = false;
 			return;
 		}
-		FARPROC psp;
-		
-		psp = GetProcAddress(pcapinst,"pcap_sendpacket");
-		if(!PacketSendPacket) PacketSendPacket =
-			(int (__cdecl *)(pcap_t *,const u_char *,int))psp;
-		
-		psp = GetProcAddress(pcapinst,"pcap_close");
-		if(!PacketClose) PacketClose =
-			(void (__cdecl *)(pcap_t *)) psp;
-		
-		psp = GetProcAddress(pcapinst,"pcap_freealldevs");
-		if(!PacketFreealldevs) PacketFreealldevs =
-			(void (__cdecl *)(pcap_if_t *)) psp;
-
-		psp = GetProcAddress(pcapinst,"pcap_open");
-		if(!PacketOpen) PacketOpen =
-			(pcap_t* (__cdecl *)(char const *,int,int,int,struct pcap_rmtauth *,char *)) psp;
-
-		psp = GetProcAddress(pcapinst,"pcap_next_ex");
-		if(!PacketNextEx) PacketNextEx = 
-			(int (__cdecl *)(pcap_t *, struct pcap_pkthdr **, const u_char **)) psp;
-
-		psp = GetProcAddress(pcapinst,"pcap_findalldevs_ex");
-		if(!PacketFindALlDevsEx) PacketFindALlDevsEx =
-			(int (__cdecl *)(char *, struct pcap_rmtauth *, pcap_if_t **, char *)) psp;
-
-		if(PacketFindALlDevsEx==0 || PacketNextEx==0 || PacketOpen==0 || 
-			PacketFreealldevs==0 || PacketClose==0 || PacketSendPacket==0) {
-			LOG_MSG("Wrong WinPcap version or something");
-			load_success = false;
-			return;
-		}
-
-#endif
 
 		// get irq and base
 		Bitu irq = (Bitu)section->Get_int("nicirq");
@@ -1586,101 +1518,6 @@ public:
 			mac[4]=macint[4]; mac[5]=macint[5];
 		}
 
-		// find out which pcap device to use
-		const char* realnicstring=section->Get_string("realnic");
-		pcap_if_t *alldevs;
-		pcap_if_t *currentdev = NULL;
-		char errbuf[PCAP_ERRBUF_SIZE];
-		unsigned int userdev;
-#ifdef WIN32
-		if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &alldevs, errbuf) == -1)
-#else
-		if (pcap_findalldevs(&alldevs, errbuf) == -1)
-#endif
- 		{
-			LOG_MSG("Cannot enumerate network interfaces: %s\n", errbuf);
-			load_success = false;
-			return;
-		}
-		if (!strcasecmp(realnicstring,"list")) {
-			// print list and quit
-			Bitu i = 0;
-			LOG_MSG("\nNetwork Interface List \n-----------------------------------");
-			for(currentdev=alldevs; currentdev!=NULL; currentdev=currentdev->next) {
-				const char* desc = "no description"; 
-				if(currentdev->description) desc=currentdev->description;
-				i++;
-				LOG_MSG("%2d. %s\n    (%s)\n",(int)i,currentdev->name,desc);
-			}
-			pcap_freealldevs(alldevs);
-			load_success = false;
-			return;
-		} else if(1==sscanf(realnicstring,"%u",&userdev)) {
-			// user passed us a number
-			Bitu i = 0;
-			currentdev=alldevs;
-			while(currentdev!=NULL) {
-				i++;
-				if(i==userdev) break;
-				else currentdev=currentdev->next;
-			}
-		} else {
-			// user might have passed a piece of name
-			for(currentdev=alldevs; currentdev!=NULL; currentdev=currentdev->next) {
-				if(strstr(currentdev->name,realnicstring)) {
-					break;
-				}else if(currentdev->description!=NULL &&
-					strstr(currentdev->description,realnicstring)) {
-					break;
-				}
-			}
-		}
-
-		if(currentdev==NULL) {
-			LOG_MSG("Unable to find network interface - check realnic parameter\n");
-			load_success = false;
-			pcap_freealldevs(alldevs);
-			return;
-		}
-		// print out which interface we are going to use
-        const char* desc = "no description"; 
-		if(currentdev->description) desc=currentdev->description;
-		LOG_MSG("Using Network interface:\n%s\n(%s)\n",currentdev->name,desc);
-		
-		// attempt to open it
-#ifdef WIN32
-		if ( (adhandle= pcap_open(
-			currentdev->name, // name of the device
-            65536,            // portion of the packet to capture
-                              // 65536 = whole packet 
-            PCAP_OPENFLAG_PROMISCUOUS,    // promiscuous mode
-            -1,             // read timeout
-            NULL,             // authentication on the remote machine
-            errbuf            // error buffer
-            ) ) == NULL)
-#else
-		/*pcap_t *pcap_open_live(const char *device, int snaplen,
-               int promisc, int to_ms, char *errbuf)*/
-		if ( (adhandle= pcap_open_live(
-			currentdev->name, // name of the device
-            65536,            // portion of the packet to capture
-                              // 65536 = whole packet 
-            true,    // promiscuous mode
-            -1,             // read timeout
-            errbuf            // error buffer
-            ) ) == NULL)
-
-#endif        
-        {
-				LOG_MSG("\nUnable to open the interface: %s.", errbuf);
-        	pcap_freealldevs(alldevs);
-			load_success = false;
-			return;
-		}
-		pcap_freealldevs(alldevs);
-#ifndef WIN32
-		pcap_setnonblock(adhandle,1,errbuf);
-#endif
 		// create the bochs NIC class
 		theNE2kDevice = new bx_ne2k_c ();
 		memcpy(theNE2kDevice->s.physaddr, mac, 6);
@@ -1698,16 +1535,18 @@ public:
 				dosbox_write,IO_MB|IO_MW);
 		}
 		TIMER_AddTickHandler(NE2000_Poller);
-	}	
+		addne2k = true;
+	}
 	
 	~NE2K() {
-		if(adhandle) pcap_close(adhandle);
-		adhandle=0;
+		if(ethernet != 0) delete ethernet;
+		ethernet=0;
 		if(theNE2kDevice != 0) delete theNE2kDevice;
 		theNE2kDevice=0;
 		TIMER_DelTickHandler(NE2000_Poller);
 		PIC_RemoveEvents(NE2000_TX_Event);
-	}	
+		if (addne2k) VFILE_Remove("NE2000.COM","SYSTEM");
+	}
 };
 
 static NE2K* test = NULL;
@@ -1740,5 +1579,3 @@ void NE2K_Init() {
 	AddExitFunction(AddExitFunctionFuncPair(NE2K_ShutDown),true);
 	AddVMEventFunction(VM_EVENT_RESET,AddVMEventFunctionFuncPair(NE2K_OnReset));
 }
-
-#endif // C_NE2000
