@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 
 #include "dosbox.h"
+#include "logging.h"
 #include "setup.h"
 #include "vga.h"
 #include "inout.h"
@@ -29,6 +30,9 @@
 extern bool vga_enable_3C6_ramdac;
 extern bool vga_sierra_lock_565;
 
+extern unsigned int vbe_window_granularity;
+extern unsigned int vbe_window_size;
+
 // Tseng ET4K data
 typedef struct {
     uint8_t extensionsEnabled;
@@ -114,7 +118,7 @@ void write_p3d5_et4k(Bitu reg,Bitu val,Bitu iolen) {
     // TODO: Bit 6 may have effect on emulation
     STORE_ET4K(3d4, 34);
 
-    case 0x35: 
+    case 0x35:
     /*
     3d4h index 35h (R/W): Overflow High
     bit    0  Vertical Blank Start Bit 10 (3d4h index 15h).
@@ -350,7 +354,7 @@ void FinishSetMode_ET4K(Bitu crtc_base, VGA_ModeExtraData* modeData) {
     // Reinterpret hor_overflow. Curiously, three bits out of four are
     // in the same places. Input has hdispend (not supported), output
     // has CRTC offset (also not supported)
-    uint8_t et4k_hor_overflow = 
+    uint8_t et4k_hor_overflow =
         (modeData->hor_overflow & 0x01) |
         (modeData->hor_overflow & 0x04) |
         (modeData->hor_overflow & 0x10);
@@ -418,7 +422,21 @@ void DetermineMode_ET4K() {
     // until I figure a way to either distinguish M_VGA and M_LIN8 or
     // merge them.
     if (vga.attr.mode_control & 1) {
-        if (vga.gfx.mode & 0x40) VGA_SetMode((et4k.biosMode<=0x13)?M_VGA:M_LIN8); // Ugly...
+        if (vga.gfx.mode & 0x40) {
+            // NTS: According to a register dump of a real Tseng ET4000, and
+            //      the datasheet, mode 0x2E (640x480 256-color) can be differentiated
+            //      from 320x200 256-color mode by whether the 8BIT bit is set
+            //      in the "mode control" register of the Attribute Controller.
+            //      If it is set, we're in standard VGA mode. If not set, but the
+            //      256-color bit is set in the Graphics Controller, then we're in
+            //      the SVGA modes.
+            if (vga.attr.mode_control & 0x40) {
+                VGA_SetMode((et4k.biosMode<=0x13)?M_VGA:M_LIN8); // Ugly...
+            }
+            else {
+                VGA_SetMode(M_LIN8);
+            }
+        }
         else if (vga.gfx.mode & 0x20) VGA_SetMode(M_CGA4);
         else if ((vga.gfx.miscellaneous & 0x0c)==0x0c) VGA_SetMode(M_CGA2);
         else VGA_SetMode((et4k.biosMode<=0x13)?M_EGA:M_LIN4);
@@ -582,6 +600,8 @@ void INT10Extensions_ET4K() {
     }
 }
 
+extern bool VGA_BIOS_use_rom;
+
 void SVGA_Setup_TsengET4K(void) {
     svga.write_p3d5 = &write_p3d5_et4k;
     svga.read_p3d5 = &read_p3d5_et4k;
@@ -629,9 +649,6 @@ void SVGA_Setup_TsengET4K(void) {
         vga.mem.memsize = 512*1024;
     else
         vga.mem.memsize = 1024*1024;
-
-    // Tseng ROM signature
-    phys_writes(PhysMake(0xc000,0)+0x0075, " Tseng ", 8);
 }
 
 
@@ -811,7 +828,12 @@ void write_p3cd_et3k(Bitu port,Bitu val,Bitu iolen) {
     (void)iolen;//UNUSED
     vga.svga.bank_write = val & 0x07;
     vga.svga.bank_read = (val>>3) & 0x07;
-    vga.svga.bank_size = (val&0x40)?64*1024:128*1024;
+
+    if (vbe_window_granularity > 0)
+        vga.svga.bank_size = vbe_window_granularity; /* allow different sizes for dev testing */
+    else
+        vga.svga.bank_size = (val&0x40)?64*1024:128*1024;
+
     VGA_SetupHandlers();
 }
 
@@ -884,7 +906,7 @@ void FinishSetMode_ET3K(Bitu crtc_base, VGA_ModeExtraData* modeData) {
     IO_Write(crtc_base,0x25);IO_Write(crtc_base+1,et4k_ver_overflow);
 
     // Clear remaining ext CRTC registers
-    for (uint8_t i=0x16; i<=0x21; i++) {
+    for (uint8_t i=0x1b; i<=0x21; i++) {
         IO_Write(crtc_base,i);
         IO_Write(crtc_base+1,0);
     }
@@ -978,16 +1000,6 @@ void SVGA_Setup_TsengET3K(void) {
     IO_RegisterWriteHandler(0x3cd,write_p3cd_et3k,IO_MB);
 
     vga.mem.memsize = 512*1024; // Cannot figure how this was supposed to work on the real card
-
-    // Tseng ROM signature
-    PhysPt rom_base=PhysMake(0xc000,0);
-    phys_writeb(rom_base+0x0075,' ');
-    phys_writeb(rom_base+0x0076,'T');
-    phys_writeb(rom_base+0x0077,'s');
-    phys_writeb(rom_base+0x0078,'e');
-    phys_writeb(rom_base+0x0079,'n');
-    phys_writeb(rom_base+0x007a,'g');
-    phys_writeb(rom_base+0x007b,' ');
 }
 
 // save state support
