@@ -49,6 +49,7 @@
 #define DBCS16_LEN 65536 * 32
 #define DBCS24_LEN 65536 * 72
 #define SBCS24_LEN 256 * 48
+#define	GAIJI_COUNT	16
 
 #if defined(LINUX) && C_X11
 static Display *font_display;
@@ -75,6 +76,11 @@ uint8_t jfont_cache_dbcs_14[65536];
 uint8_t jfont_cache_dbcs_16[65536];
 uint8_t jfont_cache_dbcs_24[65536];
 
+static uint8_t jfont_yen[32];
+static uint8_t jfont_kana[32*64];
+static uint8_t jfont_kanji[96];
+static uint16_t gaiji_seg;
+
 typedef struct {
     char id[ID_LEN];
     char name[NAME_LEN];
@@ -108,8 +114,8 @@ static bool use20pixelfont;
 extern bool autoboxdraw;
 extern bool ttf_dosv;
 #endif
-
-bool gbk = false;
+extern bool gbk, chinasea;
+extern int bootdrive;
 bool del_flag = true;
 bool yen_flag = false;
 bool jfont_init = false;
@@ -125,9 +131,11 @@ bool CodePageHostToGuestUTF16(char *d/*CROSS_LEN*/,const uint16_t *s/*CROSS_LEN*
 
 bool isKanji1(uint8_t chr) {
     if (dos.loaded_codepage == 936 || IS_PDOSV)
-        return (chr >= (gbk?0x81:0xa1) && chr <= 0xfe);
-    else if (dos.loaded_codepage == 949 || dos.loaded_codepage == 950 || IS_CDOSV || IS_KDOSV)
-        return (chr >= 0x81 && chr <= 0xfe);
+        return chr >= (gbk ? 0x81 : 0xa1) && chr <= 0xfe;
+    else if (dos.loaded_codepage == 950 || IS_CDOSV)
+        return chr >= 0x81 && chr <= 0xfe && !(!chinasea && chr >= 0xc7 && chr <= 0xc8);
+    else if (dos.loaded_codepage == 949 || IS_KDOSV)
+        return chr >= 0x81 && chr <= 0xfe;
     else
         return (chr >= 0x81 && chr <= 0x9f) || (chr >= 0xe0 && chr <= 0xfc);
 }
@@ -138,7 +146,7 @@ bool isKanji2(uint8_t chr) {
 #else
     if (dos.loaded_codepage == 936 || dos.loaded_codepage == 949 || dos.loaded_codepage == 950 || (IS_DOSV && !IS_JDOSV))
 #endif
-        return chr >= 0x40 && chr <= 0xfe;
+        return chr >= (dos.loaded_codepage == 936 && !gbk? 0xa1 : 0x40) && chr <= 0xfe;
     else
         return (chr >= 0x40 && chr <= 0x7e) || (del_flag && chr == 0x7f) || (chr >= 0x80 && chr <= 0xfc);
 }
@@ -173,7 +181,7 @@ void readfontxtbl(fontxTbl *table, Bitu size, FILE *fp)
     }
 }
 
-static bool LoadFontxFile(const char *fname, int height = 16) {
+static bool LoadFontxFile(const char *fname, int height, bool dbcs) {
     fontx_h head;
     fontxTbl *table;
     Bitu code;
@@ -199,7 +207,34 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
 #endif
 	}
 	if (getfontx2header(mfile, &head) != 0) {
-		if (dos.loaded_codepage == 936 || dos.loaded_codepage == 950) {
+        if (!dbcs) {
+            fseek(mfile, 0L, SEEK_END);
+            long int sz = ftell(mfile);
+            rewind(mfile);
+            if (sz == 256 * 15) {
+                if (height == 16)
+                    for(int i = 0 ; i < 256 ; i++) fread(&jfont_sbcs_16[i * 16], sizeof(uint8_t), 15, mfile);
+                else if(height == 19)
+                    for(int i = 0 ; i < 256 ; i++) fread(&jfont_sbcs_19[i * 19 + 2], sizeof(uint8_t), 15, mfile);
+                if (height == 16 || height == 19) {
+                    fclose(mfile);
+                    return true;
+                }
+            } else if (sz == SBCS16_LEN) {
+                if(height == 16)
+                    for(int i = 0 ; i < 256 ; i++) fread(&jfont_sbcs_16[i * 16], sizeof(uint8_t), 16, mfile);
+                else if(height == 19)
+                    for(int i = 0 ; i < 256 ; i++) fread(&jfont_sbcs_19[i * 19 + 1], sizeof(uint8_t), 16, mfile);
+                if (height == 16 || height == 19) {
+                    fclose(mfile);
+                    return true;
+                }
+            } else if ((sz == SBCS24_LEN || sz == 16384) && height == 24) {
+                fread(jfont_sbcs_24, sizeof(uint8_t), SBCS24_LEN, mfile);
+                fclose(mfile);
+                return true;
+            }
+		} else if (dos.loaded_codepage == 936 || dos.loaded_codepage == 950) {
             fseek(mfile, 0L, SEEK_END);
             long int sz = ftell(mfile);
             rewind(mfile);
@@ -209,29 +244,33 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
                 if (!fontdata14) {fclose(mfile);return false;}
                 fread(fontdata14, sizeof(uint8_t), sz, mfile);
                 fontsize14 = sizeof(uint8_t)*sz;
+                fclose(mfile);
+                return true;
             } else if (height==16) {
                 if (!sz||((sz%15)&&(sz%16))) {fclose(mfile);return false;}
                 fontdata16 = (uint8_t *)malloc(sizeof(uint8_t)*sz);
                 if (!fontdata16) {fclose(mfile);return false;}
                 fread(fontdata16, sizeof(uint8_t), sz, mfile);
                 fontsize16 = sizeof(uint8_t)*sz;
+                fclose(mfile);
+                return true;
             } else if (height==24) {
                 if (!sz||(sz%24)) {fclose(mfile);return false;}
                 fontdata24 = (uint8_t *)malloc(sizeof(uint8_t)*sz);
                 if (!fontdata24) {fclose(mfile);return false;}
                 fread(fontdata24, sizeof(uint8_t), sz, mfile);
                 fontsize24 = sizeof(uint8_t)*sz;
+                fclose(mfile);
+                return true;
             }
-            fclose(mfile);
-            return true;
         }
 		fclose(mfile);
 		LOG_MSG("MSG: no correct FONTX2 header found\n");
 		return false;
-    }
+	}
 	// switch whether the font is DBCS or not
-	if (head.type == 1) {
-		if (head.width == 14 && head.height == 14) {
+	if (head.type == 1 && dbcs) {
+		if (head.width == 14 && head.height == 14 && height == 14) {
 			size = getc(mfile);
 			table = (fontxTbl *)calloc(size, sizeof(fontxTbl));
 			readfontxtbl(table, size, mfile);
@@ -241,8 +280,7 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
 					jfont_cache_dbcs_14[code] = 1;
 				}
 			}
-		}
-		else if (head.width == 16 && head.height == 16) {
+		} else if (head.width == 16 && head.height == 16 && height == 16) {
 			size = getc(mfile);
 			table = (fontxTbl *)calloc(size, sizeof(fontxTbl));
 			readfontxtbl(table, size, mfile);
@@ -252,8 +290,7 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
 					jfont_cache_dbcs_16[code] = 1;
 				}
 			}
-		}
-		else if (head.width == 24 && head.height == 24) {
+		} else if (head.width == 24 && head.height == 24 && height == 24) {
 			size = getc(mfile);
 			table = (fontxTbl *)calloc(size, sizeof(fontxTbl));
 			readfontxtbl(table, size, mfile);
@@ -263,14 +300,13 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
 					jfont_cache_dbcs_24[code] = 1;
 				}
 			}
-		}
-		else {
+		} else {
 			fclose(mfile);
 			LOG_MSG("MSG: FONTX2 DBCS font size is not correct\n");
 			return false;
 		}
 	}
-    else {
+	else if (!dbcs) {
 		if (head.width == 8 && head.height == 19 && height == 19) {
 			fread(jfont_sbcs_19, sizeof(uint8_t), SBCS19_LEN, mfile);
 		} else if (head.width == 8 && head.height == 16) {
@@ -282,16 +318,20 @@ static bool LoadFontxFile(const char *fname, int height = 16) {
 				for(int i = 0 ; i < 256 ; i++) {
 					fread(&jfont_sbcs_19[i * 19 + 1], sizeof(uint8_t), 16, mfile);
 				}
+			} else {
+				fclose(mfile);
+				LOG_MSG("MSG: FONTX2 SBCS font size is not correct\n");
+				return false;
 			}
-		} else if (head.width == 12 && head.height == 24) {
+		} else if (head.width == 12 && head.height == 24 && height == 24) {
 			fread(jfont_sbcs_24, sizeof(uint8_t), SBCS24_LEN, mfile);
-		}
-		else {
+		} else {
 			fclose(mfile);
 			LOG_MSG("MSG: FONTX2 SBCS font size is not correct\n");
 			return false;
 		}
-    }
+	}
+	//LOG_MSG("Loaded FONTX2 file (width %d and height %d): %s\n", head.width, head.height, fname);
 	fclose(mfile);
 	return true;
 
@@ -507,7 +547,7 @@ bool GetWindowsFont(Bitu code, uint8_t *buff, int width, int height)
 		GetTextMetrics(hdc, &tm);
 		GLYPHMETRICS gm;
 		CONST MAT2 mat = { {0,1},{0,0},{0,0},{0,1} };
-		DWORD size = GetGlyphOutline(hdc, code, GGO_BITMAP, &gm, 0, NULL, &mat);
+		long size = GetGlyphOutline(hdc, code, GGO_BITMAP, &gm, 0, NULL, &mat);
 		if(size > 0) {
 			char *fontbuff = new char[size];
 			memset(fontbuff, 0, size);
@@ -547,10 +587,13 @@ bool GetWindowsFont(Bitu code, uint8_t *buff, int width, int height)
 				}
 			}
 			delete [] fontbuff;
+			SelectObject(hdc, old_font);
+			ReleaseDC(NULL, hdc);
+		} else {
+			SelectObject(hdc, old_font);
+			ReleaseDC(NULL, hdc);
+			return false;
 		}
-		SelectObject(hdc, old_font);
-		ReleaseDC(NULL, hdc);
-
 		return true;
 	}
 #endif
@@ -571,7 +614,7 @@ void GetDbcs24FrameFont(Bitu code, uint8_t *buff)
 	}
 }
 
-Bitu GetConvertedCode(Bitu code) {
+Bitu GetConvertedCode(Bitu code, int codepage) {
     char text[10];
     uint16_t uname[4];
     uname[0]=0;
@@ -581,38 +624,46 @@ Bitu GetConvertedCode(Bitu code) {
     text[2] = 0;
     if (CodePageGuestToHostUTF16(uname,text)) {
         int cp = dos.loaded_codepage;
-        dos.loaded_codepage = 932;
-        if (CodePageHostToGuestUTF16(text,uname))
+        dos.loaded_codepage = codepage;
+        if (CodePageHostToGuestUTF16(text,uname)) {
             code = (text[0] & 0xff) * 0x100 + (text[1] & 0xff);
+            dos.loaded_codepage = cp;
+            return code;
+        }
         dos.loaded_codepage = cp;
     }
-    return code;
+    return 0;
 }
 
 uint8_t *GetDbcsFont(Bitu code)
 {
 	memset(jfont_dbcs, 0, sizeof(jfont_dbcs));
 	if ((IS_JDOSV || dos.loaded_codepage == 932) && del_flag && (code & 0xFF) == 0x7F) code++;
-    if (fontdata16 && fontsize16) {
-        if (dos.loaded_codepage == 936 && !(fontsize16%16) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
-            int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 32;
-            if (offset + 32 <= fontsize16) {
-                memcpy(&jfont_dbcs_16[code * 32], fontdata16+offset, 32);
-                jfont_cache_dbcs_16[code] = 1;
-                return &jfont_dbcs_16[code * 32];
-            }
-        } else if (dos.loaded_codepage == 950 && !(fontsize16%15) && isKanji1(code/0x100)) {
-            int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
-            if (ser >= 472 && ser <= 5872) offset = (ser-472)*30;
-            else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*30+162030;
-            if (offset>-1) {
-                memcpy(&jfont_dbcs_16[code * 32], fontdata16+offset, 30);
-                jfont_cache_dbcs_16[code] = 1;
-                return &jfont_dbcs_16[code * 32];
+	if(jfont_cache_dbcs_16[code] == 0) {
+        if (fontdata16 && fontsize16) {
+            if (dos.loaded_codepage == 936 && !(fontsize16%16) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
+                int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 32;
+                if (offset + 32 <= fontsize16) {
+                    memcpy(&jfont_dbcs_16[code * 32], fontdata16+offset, 32);
+                    jfont_cache_dbcs_16[code] = 1;
+                    return &jfont_dbcs_16[code * 32];
+                }
+            } if (((dos.loaded_codepage == 936 && gbk) || dos.loaded_codepage == 950) && !(fontsize16%15) && isKanji1(code/0x100)) {
+                Bitu c = code;
+                if (dos.loaded_codepage == 936) code = GetConvertedCode(code, 950);
+                int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
+                if (code) {
+                    if (ser >= 472 && ser <= 5872) offset = (ser-472)*30;
+                    else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*30+162030;
+                }
+                code = c;
+                if (offset>-1) {
+                    memcpy(&jfont_dbcs_16[code * 32], fontdata16+offset, 30);
+                    jfont_cache_dbcs_16[code] = 1;
+                    return &jfont_dbcs_16[code * 32];
+                }
             }
         }
-    }
-	if(jfont_cache_dbcs_16[code] == 0) {
 		if(code >= 0x849f && code <= 0x84be) {
 			GetDbcsFrameFont(code, jfont_dbcs);
 			memcpy(&jfont_dbcs_16[code * 32], jfont_dbcs, 32);
@@ -621,8 +672,10 @@ uint8_t *GetDbcsFont(Bitu code)
 			memcpy(&jfont_dbcs_16[code * 32], jfont_dbcs, 32);
 			jfont_cache_dbcs_16[code] = 1;
 		} else {
-			if (!IS_JDOSV && (dos.loaded_codepage == 936 || dos.loaded_codepage == 949 || dos.loaded_codepage == 950))
-				code = GetConvertedCode(code);
+			if (!IS_JDOSV && (dos.loaded_codepage == 936 || dos.loaded_codepage == 949 || dos.loaded_codepage == 950)) {
+				code = GetConvertedCode(code, 932);
+				if (!code) return jfont_dbcs;
+			}
 			int p = NAME_LEN+ID_LEN+3;
 			uint8_t size = JPNZN16X[p++];
 			fontxTbl *table = (fontxTbl *)calloc(size, sizeof(fontxTbl));
@@ -652,34 +705,44 @@ uint8_t *GetDbcs14Font(Bitu code, bool &is14)
 {
     memset(jfont_dbcs, 0, sizeof(jfont_dbcs));
     if ((IS_JDOSV || dos.loaded_codepage == 932) && del_flag && (code & 0xFF) == 0x7F) code++;
-    if (fontdata14 && fontsize14) {
-        if (dos.loaded_codepage == 936 && !(fontsize14%14) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
-            int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 28;
-            if (offset + 28 <= fontsize14) {
-                memcpy(&jfont_dbcs_14[code * 28], fontdata14+offset, 28);
-                jfont_cache_dbcs_14[code] = 1;
-                return &jfont_dbcs_14[code * 28];
+    if(jfont_cache_dbcs_14[code] == 0) {
+        if (fontdata14 && fontsize14) {
+            if (dos.loaded_codepage == 936 && !(fontsize14%14) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
+                int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 28;
+                if (offset + 28 <= fontsize14) {
+                    memcpy(&jfont_dbcs_14[code * 28], fontdata14+offset, 28);
+                    jfont_cache_dbcs_14[code] = 1;
+                    is14 = true;
+                    return &jfont_dbcs_14[code * 28];
+                }
             }
-        } else if (dos.loaded_codepage == 950 && !(fontsize16%15) && isKanji1(code/0x100)) {
-            int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
-            if (ser >= 472 && ser <= 5872) offset = (ser-472)*30;
-            else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*30+162030;
-            if (offset>-1) {
-                memcpy(&jfont_dbcs_14[code * 28], fontdata14+offset, 28);
-                jfont_cache_dbcs_14[code] = 1;
-                return &jfont_dbcs_14[code * 28];
+            if (((dos.loaded_codepage == 936 && gbk) || dos.loaded_codepage == 950) && !(fontsize14%15) && isKanji1(code/0x100)) {
+                Bitu c = code;
+                if (dos.loaded_codepage == 936) code = GetConvertedCode(code, 950);
+                int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
+                if (code) {
+                    if (ser >= 472 && ser <= 5872) offset = (ser-472)*30;
+                    else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*30+162030;
+                }
+                code = c;
+                if (offset>-1) {
+                    memcpy(&jfont_dbcs_14[code * 28], fontdata14+offset, 28);
+                    jfont_cache_dbcs_14[code] = 1;
+                    is14 = true;
+                    return &jfont_dbcs_14[code * 28];
+                }
             }
         }
-    }
-    if(jfont_cache_dbcs_14[code] == 0) {
         if(GetWindowsFont(code, jfont_dbcs, 14, 14)) {
             memcpy(&jfont_dbcs_14[code * 28], jfont_dbcs, 28);
             jfont_cache_dbcs_14[code] = 1;
             is14 = true;
             return jfont_dbcs;
         } else {
-            if (!IS_JDOSV && (dos.loaded_codepage == 936 || dos.loaded_codepage == 949 || dos.loaded_codepage == 950))
-                code = GetConvertedCode(code);
+            if (!IS_JDOSV && (dos.loaded_codepage == 936 || dos.loaded_codepage == 949 || dos.loaded_codepage == 950)) {
+                code = GetConvertedCode(code, 932);
+                if (!code) return jfont_dbcs;
+            }
             int p = NAME_LEN+ID_LEN+3;
             uint8_t size = SHMZN14X[p++];
             fontxTbl *table = (fontxTbl *)calloc(size, sizeof(fontxTbl));
@@ -713,26 +776,25 @@ uint8_t *GetDbcs24Font(Bitu code)
 {
 	memset(jfont_dbcs, 0, sizeof(jfont_dbcs));
 	if ((IS_JDOSV || dos.loaded_codepage == 932) && del_flag && (code & 0xFF) == 0x7F) code++;
-    if (fontdata24 && fontsize24) {
-        if (dos.loaded_codepage == 936 && !(fontsize24%24) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
-            int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 72;
-            if (offset + 72 <= fontsize24) {
-                memcpy(&jfont_dbcs_24[code * 72], fontdata24+offset, 72);
-                jfont_cache_dbcs_24[code] = 1;
-                return &jfont_dbcs_24[code * 72];
-            }
-        } else if (dos.loaded_codepage == 950 && !(fontsize24%24) && isKanji1(code/0x100)) {
-            int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
-            if (ser >= 472 && ser <= 5872) offset = (ser-472)*72;
-            else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*72+162030;
-            if (offset>-1) {
-                memcpy(&jfont_dbcs_24[code * 72], fontdata24+offset, 72);
-                jfont_cache_dbcs_24[code] = 1;
-                return &jfont_dbcs_24[code * 72];
+	if(jfont_cache_dbcs_24[code] == 0) {
+        if (fontdata24 && fontsize24) {
+            if (dos.loaded_codepage == 936 && !(fontsize24%24) && (code/0x100)>0xa0 && (code/0x100)<0xff) {
+                int offset = (94 * (unsigned int)((code/0x100) - 0xa0 - 1) + ((code%0x100) - 0xa0 - 1)) * 72;
+                if (offset + 72 <= fontsize24) {
+                    memcpy(&jfont_dbcs_24[code * 72], fontdata24+offset, 72);
+                    return &jfont_dbcs_24[code * 72];
+                }
+            } else if (dos.loaded_codepage == 950 && !(fontsize24%24) && isKanji1(code/0x100)) {
+                int offset = -1, ser = (code/0x100 - 161) * 157 + ((code%0x100) - ((code%0x100)>160?161:64)) + ((code%0x100)>160?64:1);
+                if (ser >= 472 && ser <= 5872) offset = (ser-472)*72;
+                else if (ser >= 6281 && ser <= 13973) offset = (ser-6281)*72+162030;
+                if (offset>-1) {
+                    memcpy(&jfont_dbcs_24[code * 72], fontdata24+offset, 72);
+                    jfont_cache_dbcs_24[code] = 1;
+                    return &jfont_dbcs_24[code * 72];
+                }
             }
         }
-    }
-	if(jfont_cache_dbcs_24[code] == 0) {
 		if(code >= 0x809e && code < 0x80fe) {
 			if(GetWindowsFont(code - 0x807e, jfont_dbcs, 12, 24)) {
 				Bitu no, pos;
@@ -896,6 +958,7 @@ bool MakeSbcs24Font() {
 }
 
 #if defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+#include "imm.h"
 extern HWND GetHWND(void);
 bool IME_GetEnable()
 {
@@ -951,6 +1014,7 @@ void JFONT_Init() {
 #if defined(LINUX) && C_X11
 	setlocale(LC_CTYPE,"");
 #endif
+    bool reinit = jfont_init;
 	jfont_init = true;
     if (fontdata14) {
         free(fontdata14);
@@ -980,10 +1044,10 @@ void JFONT_Init() {
 #endif
 
 	Prop_path* pathprop = section->Get_path("fontxsbcs");
-	if (pathprop) {
+	if (pathprop && !reinit) {
 		std::string path=pathprop->realpath;
 		ResolvePath(path);
-		if(!LoadFontxFile(path.c_str(), 19)) {
+		if(!LoadFontxFile(path.c_str(), 19, false)) {
 			if(!MakeSbcs19Font()) {
 				LOG_MSG("MSG: SBCS 8x19 font file path is not specified.\n");
 #if defined(LINUX)
@@ -995,7 +1059,7 @@ void JFONT_Init() {
 			if(!CheckEmptyData(&jfont_sbcs_19[0x7f * 19], 19))
 				memcpy(&jfont_sbcs_19[0x5c * 19], &jfont_sbcs_19[0x7f * 19], 19);
 		}
-	} else {
+	} else if (!reinit) {
 		if(!MakeSbcs19Font())
 			LOG_MSG("MSG: SBCS 8x19 font file path is not specified.\n");
 	}
@@ -1003,23 +1067,23 @@ void JFONT_Init() {
 	if(pathprop) {
 		std::string path=pathprop->realpath;
 		ResolvePath(path);
-		LoadFontxFile(path.c_str());
+		LoadFontxFile(path.c_str(), 16, true);
 	}
 	pathprop = section->Get_path("fontxdbcs14");
 	if(pathprop) {
 		std::string path=pathprop->realpath;
 		ResolvePath(path);
-		LoadFontxFile(path.c_str(), 14);
+		LoadFontxFile(path.c_str(), 14, true);
 	}
 	if(IS_DOSV) {
 #if defined(USE_TTF)
 		autoboxdraw = true;
 #endif
 		pathprop = section->Get_path("fontxsbcs16");
-		if(pathprop) {
+		if(pathprop && !reinit) {
 			std::string path=pathprop->realpath;
 			ResolvePath(path);
-			if(!LoadFontxFile(path.c_str())) {
+			if(!LoadFontxFile(path.c_str(), 16, false)) {
 				if(!MakeSbcs16Font()) {
 					LOG_MSG("MSG: SBCS 8x16 font file path is not specified.\n");
 #if defined(LINUX)
@@ -1030,7 +1094,7 @@ void JFONT_Init() {
 				if(!CheckEmptyData(&jfont_sbcs_16[0x7f * 16], 16))
 					memcpy(&jfont_sbcs_16[0x5c * 16], &jfont_sbcs_16[0x7f * 16], 16);
 			}
-		} else {
+		} else if (!reinit) {
 			if(!MakeSbcs16Font()) {
 				LOG_MSG("MSG: SBCS 8x16 font file path is not specified.\n");
 			}
@@ -1039,13 +1103,13 @@ void JFONT_Init() {
 		if(pathprop) {
 			std::string path=pathprop->realpath;
 			ResolvePath(path);
-			LoadFontxFile(path.c_str(), 24);
+			LoadFontxFile(path.c_str(), 24, true);
 		}
 		pathprop = section->Get_path("fontxsbcs24");
-		if(pathprop) {
+		if(pathprop && !reinit) {
 			std::string path=pathprop->realpath;
 			ResolvePath(path);
-			if(!LoadFontxFile(path.c_str())) {
+			if(!LoadFontxFile(path.c_str(), 24, false)) {
 				if(!MakeSbcs24Font()) {
 					LOG_MSG("MSG: SBCS 12x24 font file path is not specified.\n");
 				}
@@ -1053,7 +1117,7 @@ void JFONT_Init() {
 				if(!CheckEmptyData(&jfont_sbcs_24[0x7f * 2 * 24], 2 * 24))
 					memcpy(&jfont_sbcs_24[0x5c * 2 * 24], &jfont_sbcs_24[0x7f * 2 * 24], 2 * 24);
 			}
-		} else {
+		} else if (!reinit) {
 			if(!MakeSbcs24Font()) {
 				LOG_MSG("MSG: SBCS 12x24 font file path is not specified.\n");
 			}
@@ -1643,3 +1707,513 @@ void DOSV_Setup()
 		}
 	}
 }
+
+#define KANJI_ROM_PAGE		(0xE0000/4096)
+
+static uint16_t j3_timer;
+static uint8_t j3_cursor_stat;
+static uint16_t j3_cursor_x;
+static uint16_t j3_cursor_y;
+static Bitu j3_text_color;
+static Bitu j3_back_color;
+static uint16_t j3_font_offset;
+static uint16_t j3_machine_code = 0;
+
+static uint16_t jis2shift(uint16_t jis)
+{
+	uint16_t high, low;
+	high = jis >> 8;
+	low = jis & 0xff;
+	if(high & 0x01) {
+		low += 0x1f;
+	} else {
+		low += 0x7d;
+	}
+	if(low >= 0x7f) {
+		low++;
+	}
+	high = ((high - 0x21) >> 1) + 0x81;
+	if(high >= 0xa0) {
+		high += 0x40;
+	}
+	return (high << 8) + low;
+}
+
+static uint16_t shift2jis(uint16_t sjis)
+{
+	uint16_t high, low;
+	high = sjis >> 8;
+	low = sjis & 0xff;
+	if(high > 0x9f) {
+		high -= 0xb1;
+	} else {
+		high -= 0x71;
+	}
+	high <<= 1;
+	high++;
+	if(low > 0x7f) {
+		low--;
+	}
+	if(low >= 0x9e) {
+		low -= 0x7d;
+		high++;
+	} else {
+		low -= 0x1f;
+	}
+	return (high << 8) + low;
+}
+
+Bitu INT60_Handler(void)
+{
+	switch (reg_ah) {
+	case 0x01:
+		reg_dx = jis2shift(reg_dx);
+		break;
+	case 0x02:
+		reg_dx = shift2jis(reg_dx);
+		break;
+	case 0x03:
+		{
+			uint16_t code = (reg_al & 0x01) ? jis2shift(reg_dx) : reg_dx;
+			SegSet16(es, 0xe000);
+			if(reg_al & 0x02) {
+				uint8_t *src = GetDbcs24Font(code);
+				uint8_t *dest = jfont_kanji;
+				for(Bitu y = 0 ; y < 24 ; y++) {
+					*dest++ = *src++;
+					*dest++ = *src++;
+					*dest++ = *src++;
+					dest++;
+				}
+				reg_si = 0x0000;
+			} else {
+				// yen
+				if(code == 0x80da) {
+					reg_si = 0x0780;
+				} else if(code >= 0x8540 && code <= 0x857E) {
+					reg_si = 0x6c20 + (code - 0x8540) * 32;
+				} else {
+					uint16_t *src = (uint16_t *)GetDbcsFont(code);
+					uint16_t *dest = (uint16_t *)jfont_kanji;
+					for(Bitu y = 0 ; y < 16 ; y++) {
+						*dest++ = *src++;
+					}
+					reg_si = 0x0000;
+				}
+			}
+			reg_al = 0;
+		}
+		break;
+	case 0x05:
+		break;
+	case 0x0c:
+		if(reg_al == 0xff) {
+			reg_al = 25 - real_readb(BIOSMEM_J3_SEG, BIOSMEM_J3_LINE_COUNT);
+		} else {
+			uint8_t line = 25 - reg_al;
+			real_writeb(BIOSMEM_J3_SEG, BIOSMEM_J3_LINE_COUNT, line);
+			line--;
+			real_writeb(BIOSMEM_SEG, BIOSMEM_NB_ROWS, line);
+		}
+		break;
+	case 0x0e:
+		SegSet16(es, GetTextSeg());
+		reg_bx = 0;
+		break;
+	case 0x0f:
+		if(reg_al == 0x00) {
+			reg_ax = 0;
+		} else if(reg_al == 0x01) {
+			DOS_ClearKeyMap();
+		}
+		break;
+	case 0x10:
+		if(reg_al == 0x00) {
+			SegSet16(es, 0xf000);
+			reg_bx = j3_font_offset;
+		}
+		break;
+	default:
+		LOG(LOG_BIOS,LOG_ERROR)("INT60:Unknown call %4X",reg_ax);
+	}
+	return CBRET_NONE;
+}
+
+Bitu INT6F_Handler(void)
+{
+	switch(reg_ah) {
+	case 0x01:
+	case 0x02:
+	case 0x03:
+	case 0x04:
+	case 0x05:
+#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
+		SDL_SetIMValues(SDL_IM_ONOFF, 1, NULL);
+#elif defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+		IME_SetEnable(TRUE);
+#endif
+		break;
+	case 0x0b:
+#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
+		SDL_SetIMValues(SDL_IM_ONOFF, 0, NULL);
+#elif defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+		IME_SetEnable(FALSE);
+#endif
+		break;
+	case 0x66:
+		{
+			reg_al = 0x00;
+#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
+			int onoff;
+			if(SDL_GetIMValues(SDL_IM_ONOFF, &onoff, NULL) == NULL) {
+				if(onoff) {
+					reg_al = 0x01;
+				}
+			}
+#elif defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+			if(IME_GetEnable()) {
+				reg_al = 0x01;
+			}
+#endif
+		}
+		break;
+	}
+	return CBRET_NONE;
+}
+
+class KanjiRomPageHandler : public PageHandler {
+private:
+	uint8_t bank;
+	Bitu GetKanji16(Bitu addr) {
+		Bitu total = bank * 0x10000 + addr;
+		uint16_t code, offset;
+		if(total < 0x10420) {
+			total -= 0x10000;
+			offset = 0x3a60;
+		} else if(total < 0x20000) {
+			total -= 0x10420;
+			offset = 0x3b21;
+		} else if(total < 0x30000) {
+			total -= 0x20000;
+			offset = 0x5020;
+		} else if(total < 0x30820) {
+			total -= 0x30000;
+			offset = 0x6540;
+		} else if(total < 0x40000) {
+			total -= 0x30820;
+			offset = 0x6621;
+		} else if(total < 0x40420) {
+			total -= 0x40000;
+			offset = 0x7a60;
+		} else if(total < 0x41be0) {
+			total -= 0x40420;
+			offset = 0x7b21;
+		} else {
+			return 0;
+		}
+		code = offset + (total / 0xc00) * 0x100 + ((total % 0xc00) / 32);
+		code = jis2shift(code);
+		GetDbcsFont(code);
+		return code * 32 + (total % 32);
+	}
+	Bitu GetKanji24(Bitu addr) {
+		Bitu total = bank * 0x10000 + addr;
+		Bitu rest;
+		uint16_t code, offset;
+		if(total >= 0x40060 && total < 0x42460) {
+			total -= 0x40060;
+			offset = 0x2021;
+		} else if(total < 0x54460) {
+			total -= 0x42460;
+			offset = 0x2121;
+		} else if(total < 0x55c00) {
+			total -= 0x54460;
+			offset = 0x2921;
+		} else if(total < 0x100000) {
+			total -= 0x58060;
+			offset = 0x3021;
+		} else {
+			return 0;
+		}
+		code = offset + (total / 0x2400) * 0x100 + (total % 0x2400) / 96;
+		code = jis2shift(code);
+		GetDbcs24Font(code);
+		rest = total % 4;
+		if(rest != 3) {
+			return code * 72 + ((total % 96) / 4) * 3 + rest;
+		}
+		return 0;
+	}
+public:
+	KanjiRomPageHandler() {
+		flags = PFLAG_HASROM;
+		bank = 0;
+	}
+	uint8_t readb(PhysPt addr) {
+		uint16_t code;
+		Bitu offset;
+		if(bank == 0) {
+			if(addr >= 0xe0000 && addr < 0xe0060) {
+				return jfont_kanji[addr - 0xe0000];
+			} else if(addr >= 0xe0780 && addr < 0xe07a0) {
+				return jfont_yen[addr - 0xe0780];
+			} else if(addr >= 0xe0c20 && addr < 0xe6be0) {
+				offset = addr - 0xe0c20;
+				code = 0x2121 + (offset / 0xc00) * 0x100 + ((offset % 0xc00) / 32);
+				code = jis2shift(code);
+				GetDbcsFont(code);
+				return jfont_dbcs_16[code * 32 + (addr % 32)];
+			} else if(addr >= 0xe6c20 && addr < 0xe7400) {
+				return jfont_kana[addr - 0xe6c20];
+			} else {
+				offset = addr - 0xe8020;
+				code = 0x3021 + (offset / 0xc00) * 0x100 + ((offset % 0xc00) / 32);
+				code = jis2shift(code);
+				GetDbcsFont(code);
+				return jfont_dbcs_16[code * 32 + (addr % 32)];
+			}
+		} else if((bank >= 1 && bank <= 3) || (bank == 4 && addr < 0xe0060)) {
+			return jfont_dbcs_16[GetKanji16(addr - 0xe0000)];
+		} else {
+			return jfont_dbcs_24[GetKanji24(addr - 0xe0000)];
+		}
+		return 0;
+	}
+	uint16_t readw(PhysPt addr) {
+		uint16_t code;
+		Bitu offset;
+		if(bank == 0) {
+			if(addr >= 0xe0000 && addr < 0xe0060) {
+				return *(uint16_t *)&jfont_kanji[addr - 0xe0000];
+			} else if(addr >= 0xe0780 && addr < 0xe07a0) {
+				return *(uint16_t *)&jfont_yen[addr - 0xe0780];
+			} else if(addr >= 0xe0c20 && addr < 0xe6be0) {
+				offset = addr - 0xe0c20;
+				code = 0x2121 + (offset / 0xc00) * 0x100 + ((offset % 0xc00) / 32);
+				code = jis2shift(code);
+				GetDbcsFont(code);
+				return *(uint16_t *)&jfont_dbcs_16[code * 32 + (addr % 32)];
+			} else if(addr >= 0xe6c20 && addr < 0xe7400) {
+				return *(uint16_t *)&jfont_kana[addr - 0xe6c20];
+			} else {
+				offset = addr - 0xe8020;
+				code = 0x3021 + (offset / 0xc00) * 0x100 + ((offset % 0xc00) / 32);
+				code = jis2shift(code);
+				GetDbcsFont(code);
+				return *(uint16_t *)&jfont_dbcs_16[code * 32 + (addr % 32)];
+			}
+		} else if((bank >= 1 && bank <= 3) || (bank == 4 && addr < 0xe1be0)) {
+			return *(uint16_t *)&jfont_dbcs_16[GetKanji16(addr - 0xe0000)];
+		} else {
+			return *(uint16_t *)&jfont_dbcs_24[GetKanji24(addr - 0xe0000)];
+		}
+		return 0;
+	}
+	uint32_t readd(PhysPt addr) {
+		return 0;
+	}
+	void writeb(PhysPt addr,uint8_t val){
+		if((val & 0x80) && val != 0xff) {
+			bank = val & 0x7f;
+		}
+	}
+	void writew(PhysPt addr,uint16_t val){
+	}
+	void writed(PhysPt addr,uint32_t val){
+	}
+};
+KanjiRomPageHandler kanji_rom_handler;
+
+uint16_t GetGaijiSeg()
+{
+	return gaiji_seg;
+}
+
+void INT60_J3_Setup()
+{
+	uint16_t code;
+
+	gaiji_seg = DOS_GetMemory(GAIJI_COUNT * 2, "J-3100 Gaiji area");
+
+	PhysPt fontdata = Real2Phys(int10.rom.font_16);
+	for(code = 0 ; code < 256 ; code++) {
+		for(int y = 0 ; y < 16 ; y++) {
+			if(code >= 0x20 && code < 0x80) {
+				phys_writeb(0xf0000 + j3_font_offset + code * 16 + y, jfont_sbcs_16[code * 16 + y]);
+				if(code == 0x5c) {
+					jfont_yen[y * 2] = jfont_sbcs_16[code * 16 + y];
+					jfont_yen[y * 2 + 1] = jfont_sbcs_16[code * 16 + y];
+				}
+			} else {
+				phys_writeb(0xf0000 + j3_font_offset + code * 16 + y, mem_readb(fontdata + code * 16 + y));
+				if(code >= 0xa1 && code <= 0xdf) {
+					jfont_kana[(code - 0xa1) * 32 + y * 2] = jfont_sbcs_16[code * 16 + y];
+					jfont_kana[(code - 0xa1) * 32 + y * 2 + 1] = jfont_sbcs_16[code * 16 + y];
+				}
+			}
+		}
+	}
+	MEM_SetPageHandler(KANJI_ROM_PAGE, 16, &kanji_rom_handler);
+}
+
+static void J3_CursorXor(Bitu x, Bitu y)
+{
+	uint16_t end = real_readb(BIOSMEM_SEG, BIOSMEM_CURSOR_TYPE);
+	uint16_t start = real_readb(BIOSMEM_SEG, BIOSMEM_CURSOR_TYPE + 1);
+	uint16_t off;
+
+	if(start != 0x20 && start <= end) {
+		y += start;
+		off = (y >> 2) * 80 + 8 * 1024 * (y & 3) + x;
+		while(start <= end) {
+			real_writeb(0xb800, off, real_readb(0xb800, off) ^ 0xff);
+			if(j3_cursor_stat == 2) {
+				real_writeb(0xb800, off + 1, real_readb(0xb800, off + 1) ^ 0xff);
+			}
+			off += 0x2000;
+			if(off >= 0x8000) {
+				off -= 0x8000;
+				off += 80;
+			}
+			start++;
+		}
+	}
+}
+
+void J3_OffCursor()
+{
+	if(j3_cursor_stat) {
+		J3_CursorXor(j3_cursor_x, j3_cursor_y);
+		j3_cursor_stat = 0;
+	}
+}
+
+void INT8_J3()
+{
+#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11) && (defined(C_SDL2) || defined(SDL_DOSBOX_X_SPECIAL))
+	SetIMPosition();
+#endif
+
+	j3_timer++;
+	if((j3_timer & 0x03) == 0) {
+		if((real_readb(BIOSMEM_J3_SEG, BIOSMEM_J3_BLINK) & 0x01) == 0 || j3_cursor_stat == 0) {
+			uint16_t x = real_readb(BIOSMEM_SEG, BIOSMEM_CURSOR_POS);
+			uint16_t y = real_readb(BIOSMEM_SEG, BIOSMEM_CURSOR_POS + 1) * 16;
+			if(j3_cursor_stat == 0) {
+				uint8_t attr = GetKanjiAttr();
+				if(attr == 0) {
+					j3_cursor_stat = 1;
+				} else {
+					j3_cursor_stat = 1;
+					if(attr == 1) {
+						j3_cursor_stat = 2;
+					}
+				}
+				j3_cursor_x = x;
+				j3_cursor_y = y;
+				J3_CursorXor(x, y);
+			} else {
+				J3_CursorXor(x, y);
+				j3_cursor_stat = 0;
+			}
+		}
+	}
+}
+
+enum J3_COLOR {
+	colorLcdBlue,
+	colorLcdWhite,
+	colorPlasma,
+	colorNormal,
+	colorMax
+};
+
+static Bitu text_color_list[colorMax] = {
+	0x3963f7, 0x3a4b51, 0xff321b, 0xffffff
+};
+
+static Bitu back_color_list[colorMax] = {
+	0xbed7d4, 0xeaf3f2, 0x6a1d22, 0x000000
+};
+
+static struct J3_MACHINE_LIST {
+	char *name;
+	uint16_t code;
+	enum J3_COLOR color;
+} j3_machine_list[] = {
+	{ "gt", 0x3130, colorPlasma },
+	{ "sgt", 0xfc27, colorPlasma },
+	{ "gx", 0xfc2d, colorPlasma },
+	{ "gl", 0xfc2b, colorLcdBlue },
+	{ "sl", 0xfc2c, colorLcdBlue },
+	{ "sgx", 0xfc26, colorPlasma },
+	{ "ss", 0x3131, colorLcdBlue },
+	{ "gs", 0xfc2a, colorLcdBlue },
+	{ "sx", 0xfc36, colorLcdBlue },
+	{ "sxb", 0xfc36, colorLcdBlue },
+	{ "sxw", 0xfc36, colorLcdWhite },
+	{ "sxp", 0xfc36, colorPlasma },
+	{ "ez", 0xfc87, colorLcdWhite },
+	{ "zs", 0xfc25, colorNormal },
+	{ "zx", 0xfc4e, colorNormal },
+	{ NULL, 0, colorMax }
+};
+
+uint16_t J3_GetMachineCode() {
+	return j3_machine_code;
+}
+
+void J3_SetType(std::string type) {
+	j3_machine_code = ConvHexWord((char *)type.c_str());
+	if(j3_machine_code == 0) j3_machine_code = 0x6a74;
+	enum J3_COLOR j3_color = colorNormal;
+	for(Bitu count = 0 ; j3_machine_list[count].name != NULL ; count++) {
+		if(type == j3_machine_list[count].name) {
+			j3_machine_code = j3_machine_list[count].code;
+			j3_color = j3_machine_list[count].color;
+			break;
+		}
+	}
+	j3_back_color = back_color_list[j3_color];
+	j3_text_color = text_color_list[j3_color];
+}
+
+void J3_GetPalette(uint8_t no, uint8_t &r, uint8_t &g, uint8_t &b)
+{
+	if(no == 0) {
+		r = (j3_back_color >> 18) & 0x3f;
+		g = (j3_back_color >> 10) & 0x3f;
+		b = (j3_back_color >> 2) & 0x3f;
+	} else {
+		r = (j3_text_color >> 18) & 0x3f;
+		g = (j3_text_color >> 10) & 0x3f;
+		b = (j3_text_color >> 2) & 0x3f;
+	}
+}
+
+bool J3_IsJapanese()
+{
+	if(IS_J3100 && real_readb(BIOSMEM_SEG, BIOSMEM_CURRENT_MODE) == 0x74) {
+		return true;
+	}
+	return false;
+}
+
+void J3_SetBiosArea(uint16_t mode)
+{
+	if(mode == 0x74) {
+		if(bootdrive < 0) {
+			real_writeb(BIOSMEM_J3_SEG, BIOSMEM_J3_MODE, 0x01);
+			if(real_readb(BIOSMEM_J3_SEG, BIOSMEM_J3_LINE_COUNT) == 0) {
+				real_writeb(BIOSMEM_J3_SEG, BIOSMEM_J3_LINE_COUNT, 25);		// line count
+			}
+			real_writew(BIOSMEM_J3_SEG, BIOSMEM_J3_CODE_SEG, GetTextSeg());
+			real_writew(BIOSMEM_J3_SEG, BIOSMEM_J3_CODE_OFFSET, 0);
+			real_writew(BIOSMEM_J3_SEG, BIOSMEM_J3_BLINK, 0x00);
+			real_writew(BIOSMEM_SEG, BIOSMEM_CURSOR_TYPE, 0x0f0f);
+		}
+		real_writeb(BIOSMEM_J3_SEG, BIOSMEM_J3_SCROLL, 0x01);		// soft scroll
+	} else {
+		real_writeb(BIOSMEM_J3_SEG, BIOSMEM_J3_MODE, 0x00);
+	}
+}
