@@ -55,7 +55,7 @@ void AUX_INT33_Takeover();
 int KEYBOARD_AUX_Active();
 void KEYBOARD_AUX_Event(float x,float y,Bitu buttons,int scrollwheel);
 extern bool MOUSE_IsLocked();
-extern bool usesystemcursor, del_flag;
+extern bool usesystemcursor, dbcs_sbcs, del_flag;
 
 bool en_int33=false;
 bool en_bios_ps2mouse=false;
@@ -613,6 +613,22 @@ extern int  user_cursor_x,  user_cursor_y;
 extern int  user_cursor_sw, user_cursor_sh;
 extern bool user_cursor_locked;
 
+/* Do not access BIOS data areas or other real-mode areas
+ * unless running in real mode or virtual 8086 mode, AND the
+ * DOS kernel must not be shutdown since INT 33h is an MS-DOS
+ * interface. That means booting into a guest OS should disable
+ * all INT 33h emulation leaving only PS/2, serial, and AUX. */
+static bool AllowINT33RMAccess() {
+	if (!dos_kernel_disabled) {
+		if (!cpu.pmode) // not protected mode
+			return true;
+		if (GETFLAG(VM)) // protected mode, but virtual 8086 mode
+			return true;
+	}
+
+	return false;
+}
+
 /* FIXME: Re-test this code */
 void Mouse_CursorMoved(float xrel,float yrel,float x,float y,bool emulate) {
     extern bool Mouse_Vertical;
@@ -650,7 +666,7 @@ void Mouse_CursorMoved(float xrel,float yrel,float x,float y,bool emulate) {
     if (emulate) {
         mouse.x += dx;
         mouse.y += dy;
-    } else if (CurMode != NULL) {
+    } else if (AllowINT33RMAccess() && CurMode != NULL) {
         if (CurMode->type == M_TEXT) {
             mouse.x = x*real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS)*8;
             mouse.y = y*(IS_EGAVGA_ARCH?(real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS)+1):25)*8;
@@ -763,10 +779,10 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
 #if defined(USE_TTF)
     ttfuse = ttf.inUse;
     ttfcols = ttf.cols;
-    if (ttfuse&&isDBCSCP()&&!(c1==0&&c2==(int)(ttf.cols-1)&&r1==0&&r2==(int)(ttf.lins-1))) {
+    if (ttfuse&&isDBCSCP()&&dbcs_sbcs&&!(c1==0&&c2==(int)(ttf.cols-1)&&r1==0&&r2==(int)(ttf.lins-1))) {
         ttf_cell *curAC = curAttrChar;
         for (unsigned int y = 0; y < ttf.lins; y++) {
-            if ((int)y>=r1&&(int)y<=r2)
+            if ((int)y>=r1&&(int)y<=r2) {
                 for (unsigned int x = 0; x < ttf.cols; x++)
                     if ((int)x>=c1&&(int)x<=c2&&curAC[rtl?ttf.cols-x-1:x].selected) {
                         if ((int)x==c1&&c1>0&&curAC[rtl?ttf.cols-x-1:x].skipped&&!curAC[rtl?ttf.cols-x-2:x-1].selected&&curAC[rtl?ttf.cols-x-2:x-1].doublewide) {
@@ -780,13 +796,19 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                             text[len++]=result;
                         }
                     }
+                while (len>0&&text[len-1]==32) text[--len]=0;
+                if (y<r2) {
+                    text[len++]='\r';
+                    text[len++]='\n';
+                }
+            }
             curAC += ttf.cols;
         }
     } else
 #endif
 	for (int i=r1; i<=r2; i++) {
         bool lead1 = false, lead2 = false;
-        if (IS_DOSV) {
+        if (IS_DOSV && (CurMode->type != M_DCGA || IS_J3100)) {
             for (int k=0; k<c1; k++) {
                 if (lead1) lead1=false;
                 else if (isKanji1(real_readb(seg,(i*c+k)*2))) lead1=true;
@@ -810,9 +832,9 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                     uint16_t prevres=mem_readw(where-(ttfuse&&rtl?-2:2));
                     if (!((prevres & 0xFF00u) != 0u && (prevres & 0xFCu) != 0x08u && prevres==result))
                         text[len++]=result;
-                } else
+                } else if (result)
                     text[len++]=result;
-            } else if (IS_DOSV) {
+            } else if (IS_DOSV && (CurMode->type != M_DCGA || IS_J3100)) {
                 if (lead2) lead2=false;
                 else if (isKanji1(real_readb(seg,(i*c+j)*2))) lead2=true;
                 result=real_readb(seg,(i*c+j)*2);
@@ -827,6 +849,7 @@ const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint
                 bool find = isJEGAEnabled()?std::find(jtbs.begin(), jtbs.end(), std::make_pair(i,j)) != jtbs.end():false;
                 if (!isJEGAEnabled()||j>c1||!find) {
                     ReadCharAttr(ttfuse&&rtl?ttfcols-j-1:j,i,page,&result);
+                    if (!result && CurMode->type == M_DCGA && !IS_J3100) result=32;
                     text[len++]=result;
                     if (isJEGAEnabled() && find && del_flag && (text[len-1]&0xFF) == 0x7F) text[len-1]++;
                 }
@@ -883,12 +906,12 @@ void Mouse_Select(int x1, int y1, int x2, int y2, int w, int h, bool select) {
 #if defined(USE_TTF)
     ttfuse = ttf.inUse;
     ttfcols = ttf.cols;
-    if (ttfuse&&(!IS_EGAVGA_ARCH||CurMode->mode!=3||isDBCSCP())) {
+    if (ttfuse&&(!IS_EGAVGA_ARCH||CurMode->mode!=3||isDBCSCP()&&dbcs_sbcs)) {
         ttf_cell *newAC = newAttrChar;
         for (unsigned int y = 0; y < ttf.lins; y++) {
             if (y>=r1&&y<=r2)
                 for (unsigned int x = 0; x < ttf.cols; x++)
-                    if ((x>=c1||((IS_PC98_ARCH||isDBCSCP())&&c1>0&&x==c1-1&&(newAC[rtl?ttf.cols-x-1:x].chr&0xFF00)&&(newAC[rtl?ttf.cols-x:x+1].chr&0xFF)==32))&&x<=c2)
+                    if ((x>=c1||((IS_PC98_ARCH||isDBCSCP()&&dbcs_sbcs)&&c1>0&&x==c1-1&&(newAC[rtl?ttf.cols-x-1:x].chr&0xFF00)&&(newAC[rtl?ttf.cols-x:x+1].chr&0xFF)==32))&&x<=c2)
                         newAC[rtl?ttf.cols-x-1:x].selected = select?1:0;
             newAC += ttf.cols;
         }
@@ -901,11 +924,11 @@ void Mouse_Select(int x1, int y1, int x2, int y2, int w, int h, bool select) {
 			if (IS_PC98_ARCH) {
 				PhysPt where = CurMode->pstart+((i*80)+j)*2;
 				mem_writeb(where+0x2000,mem_readb(where+0x2000)^16);
-			} else if (IS_DOSV) {
+			} else if (IS_DOSV && !IS_J3100) {
 				uint8_t attr = real_readb(seg,(i*c+j)*2+1);
 				real_writeb(seg,(i*c+j)*2+1,attr/0x10+(attr&0xF)*0x10);
 				if (j==c2) WriteCharTopView(c*i*2,j+1);
-			} else
+			} else if (CurMode->type != M_DCGA)
 				real_writeb(0xb800,(i*c+(ttfuse&&rtl?ttfcols-j-1:j))*2+1,real_readb(0xb800,(i*c+(ttfuse&&rtl?ttfcols-j-1:j))*2+1)^119);
 		}
 }
@@ -1788,10 +1811,10 @@ static Bitu INT74_Handler(void) {
          * the original DOSBox code did it. Doing this allows the INT 33h emulation
          * to draw the cursor while not causing Windows 3.1 to crash or behave
          * erratically. */
-        if (en_int33) DrawCursor();
+        if (AllowINT33RMAccess() && en_int33) DrawCursor();
 
         /* Check for an active Interrupt Handler that will get called */
-        if (mouse.sub_mask & mouse.event_queue[mouse.events].type) {
+        if (AllowINT33RMAccess() && (mouse.sub_mask & mouse.event_queue[mouse.events].type)) {
             reg_ax=mouse.event_queue[mouse.events].type;
             reg_bx=mouse.event_queue[mouse.events].buttons;
             reg_cx=(uint16_t)POS_X;
@@ -1936,8 +1959,11 @@ void MOUSE_Startup(Section *sec) {
     Section_prop *section=static_cast<Section_prop *>(control->GetSection("dos"));
 	Section_prop * pc98_section=static_cast<Section_prop *>(control->GetSection("pc98"));
     RealPt i33loc=0;
+    extern bool serialMouseEmulated;
 
-    /* TODO: Needs to check for mouse, and fail to do anything if neither PS/2 nor serial mouse emulation enabled */
+    if (MouseTypeNone() && !serialMouseEmulated) {
+        return;
+    }
 
     en_int33_hide_if_intsub=section->Get_bool("int33 hide host cursor if interrupt subroutine");
 
