@@ -778,6 +778,74 @@ public:
 
 };
 
+class Timer;
+
+/// Timer callback type
+struct Timer_Callback {
+public:
+		/// The timer has expired.
+		/** Callbacks for timers take one parameter, the number of ticks since
+		 *  application start. Note that this value may wrap after a little less
+		 *  than 500 days. If you want callbacks to be called again, return the
+		 *  delay in ticks relative to the scheduled time of this
+		 *  callback (which may be earlier than now() ). Otherwise return 0. */
+		virtual Ticks timerExpired(Ticks time) = 0;
+		virtual ~Timer_Callback() {}
+};
+
+/** \brief Timing service.
+ *  Time is measured in ticks. A tick is about 10 msec.
+ *
+ *  Note that this is not suitable as a high-accuracy timing service. Timer
+ *  events can be off by many ticks, and a tick may be slightly more or
+ *  slightly less than 10 msec. Because of that, Timers are only intended for
+ *  simple animation effects, input timeouts and similar things.
+ *
+ */
+class Timer {
+protected:
+	/// Number of ticks since application start.
+	static Ticks ticks;
+
+	/// Compare two integers for 'less-than'.
+	struct ltuint { bool operator()(Ticks i1, Ticks i2) const {
+		return (i1 < i2);
+	} };
+
+	/// Active timers.
+	static std::multimap<Ticks,Timer_Callback*,ltuint> timers;
+
+public:
+	/// Advance time and check for expired timers.
+	static void check(Ticks ticks);
+	static void check_to(Ticks ticks);
+
+	/// Add a timed callback. \p ticks is a value relative to now().
+	/** \p cb is not copied. */
+	static void add(Timer_Callback *cb, const Ticks ticks) { timers.insert(std::pair<const Ticks,Timer_Callback *>(ticks+Timer::ticks,cb)); }
+
+	static void remove(const Timer_Callback *const timer);
+
+	/// Return current time (ticks since application start)
+	static Ticks now() { return ticks; }
+
+	/// Return number of ticks until next scheduled timer or 0 if no timers
+	static Ticks next();
+};
+
+struct vscrollbarlayout {
+    SDL_Rect scrollthumbRegion = {0,0,0,0};
+    SDL_Rect scrollbarRegion = {0,0,0,0};
+    int thumbwidth = 0;
+    int thumbheight = 0;
+    int thumbtravel = 0;
+    bool drawthumb = false;
+    bool disabled = true;
+    bool draw = false;
+    int xleft = -1;
+    int ytop = -1;
+};
+
 /* Window wrapper to make scrollable regions */
 class WindowInWindow : public Window {
 protected:
@@ -804,6 +872,13 @@ public:
 	/// Key was pressed. Returns true if event was handled.
 	virtual bool keyDown(const Key &key);
 
+	virtual void getVScrollInfo(vscrollbarlayout &vsl) const;
+	virtual void paintScrollBarArrowInBox(Drawable &dscroll,const int x,const int y,const int w,const int h,bool downArrow,bool disabled) const;
+	virtual void paintScrollBarThumbDragOutline(Drawable &dscroll,const vscrollbarlayout &vsl) const;
+	virtual void paintScrollBarBackground(Drawable &dscroll,const vscrollbarlayout &vsl) const;
+	virtual void paintScrollBarThumb(Drawable &dscroll, vscrollbarlayout &vsl) const;
+	virtual void paintScrollBar3DOutset(Drawable &dscroll, int x, int y, int w, int h) const;
+	virtual void paintScrollBar3DInset(Drawable &dscroll, int x, int y, int w, int h) const;
 	virtual void paintAll(Drawable &d) const;
 
 	virtual void resize(int w, int h);
@@ -813,9 +888,32 @@ public:
 
     bool    hscroll_dragging = false;
     bool    vscroll_dragging = false;
+    bool    vscroll_uparrowhold = false;
+    bool    vscroll_downarrowhold = false;
+    bool    vscroll_uparrowdown = false;
+    bool    vscroll_downarrowdown = false;
+
+    /// Timer callback type
+    struct DragTimer_Callback : public Timer_Callback {
+        public:
+            /// The timer has expired.
+            /** Callbacks for timers take one parameter, the number of ticks since
+             *  application start. Note that this value may wrap after a little less
+             *  than 500 days. If you want callbacks to be called again, return the
+             *  delay in ticks relative to the scheduled time of this
+             *  callback (which may be earlier than now() ). Otherwise return 0. */
+            virtual Ticks timerExpired(Ticks time);
+            virtual ~DragTimer_Callback() {}
+        public:
+            WindowInWindow *wnd = NULL;
+    };
 
     bool    dragging = false;
     int     drag_x = 0, drag_y = 0;
+    Ticks   drag_start = 0;
+    int     drag_start_pos = 0;
+    Timer   drag_timer;
+    DragTimer_Callback drag_timer_cb;
 
     int     scroll_pos_x = 0;
     int     scroll_pos_y = 0;
@@ -901,61 +999,6 @@ public:
 	virtual void paint(Drawable &d) const;
 };
 
-class Timer;
-
-/// Timer callback type
-struct Timer_Callback {
-public:
-		/// The timer has expired.
-		/** Callbacks for timers take one parameter, the number of ticks since
-		 *  application start. Note that this value may wrap after a little less
-		 *  than 500 days. If you want callbacks to be called again, return the
-		 *  delay in ticks relative to the scheduled time of this
-		 *  callback (which may be earlier than now() ). Otherwise return 0. */
-		virtual Ticks timerExpired(Ticks time) = 0;
-		virtual ~Timer_Callback() {}
-};
-
-/** \brief Timing service.
- *  Time is measured in ticks. A tick is about 10 msec.
- *
- *  Note that this is not suitable as a high-accuracy timing service. Timer
- *  events can be off by many ticks, and a tick may be slightly more or
- *  slightly less than 10 msec. Because of that, Timers are only intended for
- *  simple animation effects, input timeouts and similar things.
- *
- */
-class Timer {
-protected:
-	/// Number of ticks since application start.
-	static Ticks ticks;
-
-	/// Compare two integers for 'less-than'.
-	struct ltuint { bool operator()(Ticks i1, Ticks i2) const {
-		return (i1 < i2);
-	} };
-
-	/// Active timers.
-	static std::multimap<Ticks,Timer_Callback*,ltuint> timers;
-
-public:
-	/// Advance time and check for expired timers.
-	static void check(Ticks ticks);
-	static void check_to(Ticks ticks);
-
-	/// Add a timed callback. \p ticks is a value relative to now().
-	/** \p cb is not copied. */
-	static void add(Timer_Callback *cb, const Ticks ticks) { timers.insert(std::pair<const Ticks,Timer_Callback *>(ticks+Timer::ticks,cb)); }
-
-	static void remove(const Timer_Callback *const timer);
-
-	/// Return current time (ticks since application start)
-	static Ticks now() { return ticks; }
-
-	/// Return number of ticks until next scheduled timer or 0 if no timers
-	static Ticks next();
-};
-
 
 /** \brief A 24 bit per pixel RGB framebuffer aligned to 32 bit per pixel.
  *
@@ -1009,6 +1052,9 @@ protected:
 	/// time of last click for double-click detection.
 	Ticks lastclick,lastdown;
 
+    /// Integer scaling factor.
+	int scale;
+
 public:
 
 	/** Initialize SDL screen with a surface
@@ -1016,7 +1062,7 @@ public:
 	 *  The dimensions of this surface will define the screen dimensions. Changing the surface
 	 *  later on will not change the available area.
 	 */
-	ScreenSDL(SDL_Surface *surface);
+	ScreenSDL(SDL_Surface *surface, int scale);
     virtual ~ScreenSDL();
 
 	/** Change current surface
@@ -1037,13 +1083,13 @@ public:
 	Ticks update(Ticks ticks);
 
 	/// Process an SDL event. Returns \c true if event was handled.
-	bool event(const SDL_Event *ev) { return event(*ev); }
+	bool event(SDL_Event *ev) { return event(*ev); }
 
 	void watchTime();
 	Uint32 getTime() { return current_time; }
 
 	/// Process an SDL event. Returns \c true if event was handled.
-	bool event(const SDL_Event& event);
+	bool event(SDL_Event& event);
 };
 #endif
 
@@ -1786,7 +1832,7 @@ public:
 	/// Menu callback function
 	virtual void actionExecuted(ActionEventSource *src, const String &item) {
         (void)src;
-		if (item == String("Close")) close();
+		if (item == String(MSG_Get("CLOSE"))) close();
 	}
 
 	/// Add a window event handler.
@@ -2034,10 +2080,13 @@ protected:
             }
 
             selected++;
-		}
+        }
 
-		if (selected >= 0 && items[(unsigned int)selected].size() == 0) selected = -1;
-	}
+        if(selected > (int)items.size() - 1)
+            selected = items.size() - 1;
+
+        if(selected >= 0 && items[(unsigned int)selected].size() == 0) selected = -1;
+    }
 
 	virtual Size getPreferredWidth() {
 		Size width = 0,px = 0;
@@ -2557,7 +2606,7 @@ public:
 		ToplevelWindow(parent, x, y, width, 1, title) {
         wiw = new WindowInWindow(this, 5, 5, width-border_left-border_right-10, 70);
 		message = new Label(wiw, 0, 0, text, width-border_left-border_right-10);
-		close = new GUI::Button(this, (width-border_left-border_right-70)/2, 10, "Close", 70);
+		close = new GUI::Button(this, (width-border_left-border_right-70)/2, 10, MSG_Get("CLOSE"), 70);
 		close->addActionHandler(this);
 		setText(text);
 
@@ -2614,6 +2663,77 @@ public:
     }
 };
 
+class MessageBox3 : public GUI::ToplevelWindow {
+protected:
+	Label *message;
+	Button *update, *close;
+    WindowInWindow *wiw;
+public:
+	/// Create a new message box
+	template <typename STR> MessageBox3(Screen *parent, int x, int y, int width, const STR title, const STR text) :
+		ToplevelWindow(parent, x, y, width, 1, title) {
+		wiw = new WindowInWindow(this, 5, 5, width-border_left-border_right-10, 70);
+		message = new Label(wiw, 0, 0, text, width-border_left-border_right-10);
+		update = new GUI::Button(this, (width-border_left-border_right-30-70*2)/2, 10, MSG_Get("UPDATE"), 70);
+		update->addActionHandler(this);
+		close = new GUI::Button(this, (width-border_left-border_right-10)/2, 10, MSG_Get("CLOSE"), 70);
+		close->addActionHandler(this);
+		setText(text);
+
+		close->raise(); /* make sure keyboard focus is on the close button */
+		this->raise(); /* make sure THIS WINDOW has the keyboard focus */
+	}
+
+	/// Set a new text. Size of the box is adjusted accordingly.
+	template <typename STR> void setText(const STR text) {
+        int sfh;
+        int msgw;
+        bool scroll = true;
+
+        msgw = width-border_left-border_right-10;
+        message->resize(msgw, message->getHeight());
+		message->setText(text);
+
+        {
+            Screen *s = getScreen();
+            sfh = s->getHeight() - 70 - border_top - border_bottom;
+            if (sfh > (15+message->getHeight())) {
+                sfh = (15+message->getHeight());
+                scroll = false;
+            }
+        }
+
+        wiw->enableBorder(scroll);
+        wiw->enableScrollBars(false/*h*/,scroll/*v*/);
+        if (scroll) {
+            msgw -= wiw->vscroll_display_width;
+            msgw -= 2/*border*/;
+            message->resize(msgw, message->getHeight());
+        }
+
+		update->move((width-border_left-border_right-30-70*2)/2, sfh);
+		close->move((width-border_left-border_right-10)/2, sfh);
+        wiw->resize(width-border_left-border_right-10, sfh-10);
+		resize(width, sfh+close->getHeight()+border_bottom+border_top+5);
+	}
+
+	virtual bool keyDown(const GUI::Key &key) {
+        if (GUI::ToplevelWindow::keyDown(key)) return true;
+        return false;
+    }
+
+	virtual bool keyUp(const GUI::Key &key) {
+        if (GUI::ToplevelWindow::keyUp(key)) return true;
+
+        if (key.special == GUI::Key::Escape) {
+            close->executeAction();
+            return true;
+        }
+
+        return false;
+    }
+};
+
 extern int titlebar_y_start;
 extern int titlebar_y_stop;
 
@@ -2633,7 +2753,7 @@ template <typename STR> ToplevelWindow::ToplevelWindow(Screen *parent, int x, in
 	systemMenu->addItem("Restore");
 	systemMenu->addItem("");
 #endif
-	systemMenu->addItem("Close");
+	systemMenu->addItem(MSG_Get("CLOSE"));
 	systemMenu->addActionHandler(this);
     toplevel = true;
 }
