@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,13 @@
 #include "dosbox.h"
 #include "cross.h"
 #include "support.h"
+#include "dos_inc.h"
 #include <string>
 #include <limits.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <regex>
 
 #if defined(MACOSX)
 std::string MacOSXEXEPath;
@@ -36,7 +38,12 @@ std::string MacOSXResPath;
 #define _WIN32_IE 0x0400
 #endif
 #include <shlobj.h>
+typedef wchar_t host_cnv_char_t;
+#else
+typedef char host_cnv_char_t;
 #endif
+extern bool hidenonrep;
+extern char *CodePageHostToGuest(const host_cnv_char_t *s);
 
 #if defined HAVE_SYS_TYPES_H && defined HAVE_PWD_H
 #include <sys/types.h>
@@ -47,6 +54,40 @@ std::string MacOSXResPath;
 #define _mkdir(x) mkdir(x)
 #endif
 
+int resolveopt = 1;
+void autoExpandEnvironmentVariables(std::string & text, bool dosvar) {
+    static std::regex env(dosvar?"\\%([^%]+)%":"\\$\\{([^}]+)\\}");
+    std::smatch match;
+    while (std::regex_search(text, match, env)) {
+        const char * s = getenv(match[1].str().c_str());
+        const std::string var(s == NULL ? "" : s);
+        text.replace(static_cast<size_t>(match.position(0)), static_cast<size_t>(match.length(0)), var);
+    }
+}
+
+// Resolve environment variables (%VAR% [DOS/Windows] or ${VAR} [Linux/macOS]), and tilde (~) in Linux/macOS
+void ResolvePath(std::string& in) {
+    if (!resolveopt) return;
+#if defined(WIN32)
+    if (resolveopt==3) return;
+    if (resolveopt==2) {autoExpandEnvironmentVariables(in, true);return;}
+    char path[300],temp[300],*tempd=temp;
+    strcpy(tempd, in.c_str());
+    if (strchr(tempd, '%')&&ExpandEnvironmentStrings(tempd,path,300))
+        tempd=path;
+    in=std::string(tempd);
+#elif defined(OS2)
+    if (resolveopt==3) return;
+    autoExpandEnvironmentVariables(in, true);
+#else
+    struct stat test;
+    if (stat(in.c_str(),&test))
+        Cross::ResolveHomedir(in);
+    if (resolveopt==3) return;
+    autoExpandEnvironmentVariables(in, resolveopt==2);
+#endif
+}
+
 #if defined(WIN32) && !defined(HX_DOS)
 static void W32_ConfDir(std::string& in,bool create) {
 	int c = create?1:0;
@@ -188,8 +229,7 @@ bool Cross::IsPathAbsolute(std::string const& in) {
 }
 
 #if defined (WIN32)
-typedef wchar_t host_cnv_char_t;
-extern char *CodePageHostToGuest(const host_cnv_char_t *s);
+extern bool isDBCSCP();
 
 /* does the filename fit the 8.3 format? */
 static bool is_filename_8by3w(const wchar_t* fname) {
@@ -200,7 +240,7 @@ static bool is_filename_8by3w(const wchar_t* fname) {
     i=0;
     while (*fname != 0 && *fname != L'.') {
 		if (*fname<=32||*fname==127||*fname==L'"'||*fname==L'+'||*fname==L'='||*fname==L','||*fname==L';'||*fname==L':'||*fname==L'<'||*fname==L'>'||*fname==L'|'||*fname==L'?'||*fname==L'*') return false;
-		if (IS_PC98_ARCH && (*fname & 0xFF00u) != 0u && (*fname & 0xFCu) != 0x08u) i++;
+		if ((IS_PC98_ARCH || isDBCSCP()) && (*fname & 0xFF00u) != 0u && (*fname & 0xFCu) != 0x08u) i++;
 		fname++; i++; 
 	}
     if (i > 8) return false;
@@ -211,7 +251,7 @@ static bool is_filename_8by3w(const wchar_t* fname) {
     i=0;
     while (*fname != 0 && *fname != L'.') {
 		if (*fname<=32||*fname==127||*fname==L'"'||*fname==L'+'||*fname==L'='||*fname==L','||*fname==L';'||*fname==L':'||*fname==L'<'||*fname==L'>'||*fname==L'|'||*fname==L'?'||*fname==L'*') return false;
-		if (IS_PC98_ARCH && (*fname & 0xFF00u) != 0u && (*fname & 0xFCu) != 0x08u) i++;
+		if ((IS_PC98_ARCH || isDBCSCP()) && (*fname & 0xFF00u) != 0u && (*fname & 0xFCu) != 0x08u) i++;
 		fname++; i++;
 	}
     if (i > 3) return false;
@@ -264,10 +304,10 @@ bool read_directory_firstw(dir_information* dirp, wchar_t* entry_name, wchar_t*
     if (!dirp->wide) return false;
 
     // TODO: offer a config.h option to opt out of Windows widechar functions
-	dirp->handle = FindFirstFileW(dirp->wbase_path(), &dirp->search_data.w);
-	if (INVALID_HANDLE_VALUE == dirp->handle) {
-		return false;
-	}
+	do {
+		dirp->handle = FindFirstFileW(dirp->wbase_path(), &dirp->search_data.w);
+		if (INVALID_HANDLE_VALUE == dirp->handle) return false;
+	} while (hidenonrep&&CodePageHostToGuest(dirp->search_data.w.cFileName)==NULL);
 
 	// TODO: offer a config.h option to opt out of Windows widechar functions
 	wcsncpy(entry_name,dirp->search_data.w.cFileName,(MAX_PATH<CROSS_LEN)?MAX_PATH:CROSS_LEN);
@@ -286,8 +326,11 @@ bool read_directory_nextw(dir_information* dirp, wchar_t* entry_name, wchar_t* e
     if (!dirp->wide) return false;
 
     // TODO: offer a config.h option to opt out of Windows widechar functions
-	int result = FindNextFileW(dirp->handle, &dirp->search_data.w);
-	if (result==0) return false;
+	int result;
+	do {
+		result = FindNextFileW(dirp->handle, &dirp->search_data.w);
+		if (result==0) return false;
+	} while (hidenonrep&&CodePageHostToGuest(dirp->search_data.w.cFileName)==NULL);
 
     // TODO: offer a config.h option to opt out of Windows widechar functions
 	wcsncpy(entry_name,dirp->search_data.w.cFileName,(MAX_PATH<CROSS_LEN)?MAX_PATH:CROSS_LEN);
@@ -358,10 +401,11 @@ dir_information* open_directory(const char* dirname) {
 
 bool read_directory_first(dir_information* dirp, char* entry_name, char* entry_sname, bool& is_directory) {	
 	if (!dirp) return false;
-	struct dirent* dentry = readdir(dirp->dir);
-	if (dentry==NULL) {
-		return false;
-	}
+	struct dirent* dentry;
+	do {
+		dentry = readdir(dirp->dir);
+		if (dentry==NULL) return false;
+	} while (hidenonrep&&CodePageHostToGuest(dentry->d_name)==NULL);
 
 //	safe_strncpy(entry_name,dentry->d_name,(FILENAME_MAX<MAX_PATH)?FILENAME_MAX:MAX_PATH);	// [include stdio.h], maybe pathconf()
 	safe_strncpy(entry_name,dentry->d_name,CROSS_LEN);
@@ -391,16 +435,16 @@ bool read_directory_first(dir_information* dirp, char* entry_name, char* entry_s
 
 bool read_directory_next(dir_information* dirp, char* entry_name, char* entry_sname, bool& is_directory) {
 	if (!dirp) return false;
-	struct dirent* dentry = readdir(dirp->dir);
-	if (dentry==NULL) {
-		return false;
-	}
+	struct dirent* dentry;
+	do {
+		dentry = readdir(dirp->dir);
+		if (dentry==NULL) return false;
+	} while (hidenonrep&&CodePageHostToGuest(dentry->d_name)==NULL);
 
 //	safe_strncpy(entry_name,dentry->d_name,(FILENAME_MAX<MAX_PATH)?FILENAME_MAX:MAX_PATH);	// [include stdio.h], maybe pathconf()
 	safe_strncpy(entry_name,dentry->d_name,CROSS_LEN);
 	entry_sname[0]=0;
 
-
 #ifdef DIRENT_HAS_D_TYPE
 	if(dentry->d_type == DT_DIR) {
 		is_directory = true;
