@@ -37,7 +37,7 @@ extern bool dos_kernel_disabled, force_conversion;
 int msgcodepage = 0, FileDirExistUTF8(std::string &localname, const char *name);
 bool morelen = false, loadlang = false, systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 bool isSupportedCP(int newCP), CodePageHostToGuestUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), CodePageGuestToHostUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
-void InitFontHandle(void), ShutFontHandle(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname);
+void InitFontHandle(void), ShutFontHandle(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname), JFONT_Init();
 std::string langname = "", langnote = "", GetDOSBoxXPath(bool withexe=false);
 
 #define LINE_IN_MAXLEN 2048
@@ -83,7 +83,7 @@ bool InitCodePage() {
             return true;
         }
         Section_prop *section = static_cast<Section_prop *>(control->GetSection("config"));
-        if (section!=NULL && !control->opt_noconfig) {
+        if (section!=NULL && !control->opt_noconfig && !IS_PC98_ARCH && !IS_JEGA_ARCH && !IS_DOSV) {
             char *countrystr = (char *)section->Get_string("country"), *r=strchr(countrystr, ',');
             if (r!=NULL && *(r+1)) {
                 int cp = atoi(trim(r+1));
@@ -173,6 +173,11 @@ void LoadMessageFile(const char * fname) {
                     if (!strcmp(p, "CODEPAGE")) {
                         int c = atoi(r+1);
                         if ((!res || control->opt_langcp) && c>0 && isSupportedCP(c)) {
+                            if (((IS_PC98_ARCH||IS_JEGA_ARCH) && c!=437 && c!=932 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the Japanese PC-98 or JEGA/AX system.\n\nAre you sure to use the language file for this machine type?", "yesno","question", 2)) || (((IS_JDOSV && c!=932) || (IS_PDOSV && c!=936) || (IS_KDOSV && c!=949) || (IS_CDOSV && c!=950)) && c!=437 && !systemmessagebox("DOSBox-X language file", "You have specified a language file which uses a code page incompatible with the current DOS/V system.\n\nAre you sure to use the language file for this system type?", "yesno","question", 2))) {
+                                fclose(mfile);
+                                dos.loaded_codepage = cp;
+                                return;
+                            }
                             msgcodepage = c;
                             dos.loaded_codepage = c;
                         }
@@ -233,6 +238,7 @@ void LoadMessageFile(const char * fname) {
         if (msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950) {
             dos.loaded_codepage = msgcodepage;
             InitFontHandle();
+            JFONT_Init();
             dos.loaded_codepage = cp;
         }
     }
