@@ -55,7 +55,7 @@ Bitu call_program;
 extern const char *modifier;
 extern std::string langname, configfile;
 extern int enablelfn, paste_speed, wheel_key, freesizecap, wpType, wpVersion, wpBG, wpFG, lastset, blinkCursor;
-extern bool dos_kernel_disabled, force_nocachedir, wpcolon, lockmount, enable_config_as_shell_commands, load, winrun, winautorun, startcmd, startwait, startquiet, starttranspath, mountwarning, wheel_guest, clipboard_dosapi, noremark_save_state, force_load_state, sync_time, manualtime, showbold, showital, showline, showsout, char512, printfont, rtl, dbcs_sbcs, autoboxdraw, halfwidthkana, ticksLocked;
+extern bool dos_kernel_disabled, force_nocachedir, wpcolon, lockmount, enable_config_as_shell_commands, load, winrun, winautorun, startcmd, startwait, startquiet, starttranspath, mountwarning, wheel_guest, clipboard_dosapi, noremark_save_state, force_load_state, sync_time, manualtime, showbold, showital, showline, showsout, char512, printfont, rtl, gbk, chinasea, dbcs_sbcs, autoboxdraw, halfwidthkana, ticksLocked, usecon, enable_dbcs_tables;
 
 /* This registers a file on the virtual drive and creates the correct structure for it*/
 
@@ -94,8 +94,9 @@ public:
 };
 
 static std::vector<InternalProgramEntry*> internal_progs;
-void EMS_Startup(Section* sec), EMS_DoShutDown(), resetFontSize(), UpdateDefaultPrinterFont();
-void DOSBOX_UnlockSpeed2( bool pressed ), GFX_ForceRedrawScreen(void), SetWindowTransparency(int trans);
+bool isDBCSCP(void), CheckBoxDrawing(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
+void EMS_DoShutDown(void), UpdateDefaultPrinterFont(void), GFX_ForceRedrawScreen(void), resetFontSize(void), ttf_reset_colors(void), makestdcp950table(void);
+void EMS_Startup(Section* sec), DOSV_SetConfig(Section_prop *section), DOSBOX_UnlockSpeed2(bool pressed), RebootLanguage(std::string filename, bool confirm=false), SetWindowTransparency(int trans), SetOutputSwitch(const char *outputstr), runSerial(const char *str), runParallel(const char *str);
 
 void PROGRAMS_Shutdown(void) {
 	LOG(LOG_MISC,LOG_DEBUG)("Shutting down internal programs list");
@@ -233,9 +234,11 @@ void Program::WriteOut(const char * format,...) {
 //	DOS_WriteFile(STDOUT,(uint8_t *)buf,&size);
 }
 
-void Program::WriteOut_NoParsing(const char * format) {
-	uint16_t size = (uint16_t)strlen(format);
-	char const* buf = format;
+void Program::WriteOut(const char *format, const char *arguments) {
+	char buf[2048];
+	sprintf(buf,format,arguments);
+
+	uint16_t size = (uint16_t)strlen(buf);
 	dos.internal_output=true;
 	for(uint16_t i = 0; i < size;i++) {
 		uint8_t out;uint16_t s=1;
@@ -247,6 +250,41 @@ void Program::WriteOut_NoParsing(const char * format) {
 	}
 	dos.internal_output=false;
 
+//	DOS_WriteFile(STDOUT,(uint8_t *)buf,&size);
+}
+
+int Program::WriteOut_NoParsing(const char * format, bool dbcs) {
+	uint16_t size = (uint16_t)strlen(format);
+	char const* buf = format;
+	char last2 = 0, last3 = 0;
+	int lastcol = 0, COLS=IS_PC98_ARCH?80:real_readw(BIOSMEM_SEG,BIOSMEM_NB_COLS);
+	uint8_t page=usecon?real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE):0;
+	bool lead=false;
+	dos.internal_output=true;
+	int rcount = 0;
+	for(uint16_t i = 0; i < size;i++) {
+		uint8_t out;uint16_t s=1;
+		BIOS_NCOLS;
+		if (!CURSOR_POS_COL(page)) last2=last3=0;
+		if (lead) lead = false;
+		else if ((IS_PC98_ARCH || isDBCSCP()) && dbcs_sbcs && dbcs && isKanji1(buf[i])) lead = true;
+		if (buf[i] == 0xA) {
+			if (last_written_character != 0xD) {out = 0xD;DOS_WriteFile(STDOUT,&out,&s);}
+			if (usecon) rcount++;
+		} else if (usecon && lead && CURSOR_POS_COL(page)==COLS-1 && !CheckBoxDrawing(last3, last2, last_written_character, buf[i])) {
+			out = 0xD;DOS_WriteFile(STDOUT,&out,&s);
+			out = 0xA;DOS_WriteFile(STDOUT,&out,&s);
+			rcount++;
+		} else if (usecon && !CURSOR_POS_COL(page) && lastcol == COLS-1)
+			rcount++;
+		lastcol=CURSOR_POS_COL(page);
+		last3=last2;last2=last_written_character;
+		last_written_character = (char)(out = (uint8_t)buf[i]);
+		DOS_WriteFile(STDOUT,&out,&s);
+	}
+	dos.internal_output=false;
+	return rcount;
+
 //	DOS_WriteFile(STDOUT,(uint8_t *)format,&size);
 }
 
@@ -1242,11 +1280,55 @@ void CONFIG::Run(void) {
 					WriteOut(MSG_Get("PROGRAM_CONFIG_NO_PROPERTY"), pvars[1].c_str(),pvars[0].c_str());
 				return;
 			}
+            bool applynew=false;
 			Property *p = static_cast<Section_prop *>(sec2)->Get_prop(pvars[1]);
 			if ((p==NULL||p->getChange()==Property::Changeable::OnlyAtStart)&&presult!=P_SETFORCE) {
 				WriteOut(MSG_Get("PROGRAM_CONFIG_HLP_NOCHANGE"));
-				return;
+first_1:
+				WriteOut(MSG_Get("PROGRAM_CONFIG_APPLY_RESTART"));
+first_2:
+                uint8_t c;uint16_t n=1;
+                DOS_ReadFile (STDIN,&c,&n);
+                do switch (c) {
+                    case 'n':			case 'N':
+                    {
+                        DOS_WriteFile (STDOUT,&c, &n);
+                        DOS_ReadFile (STDIN,&c,&n);
+                        do switch (c) {
+                            case 0xD: WriteOut("\n");goto next;
+                            case 0x03: goto next;
+                            case 0x08: WriteOut("\b \b"); goto first_2;
+                        } while (DOS_ReadFile (STDIN,&c,&n));
+                    }
+                    case 'y':			case 'Y':
+                    {
+                        DOS_WriteFile (STDOUT,&c, &n);
+                        DOS_ReadFile (STDIN,&c,&n);
+                        do switch (c) {
+                            case 0xD: WriteOut("\n"); applynew = true; goto next;
+                            case 0x03: goto next;
+                            case 0x08: WriteOut("\b \b"); goto first_2;
+                        } while (DOS_ReadFile (STDIN,&c,&n));
+                    }
+                    case 0xD: WriteOut("\n"); goto first_1;
+                    case 0x03: goto next;
+                    case '\t':
+                    case 0x08:
+                        goto first_2;
+                    default:
+                    {
+                        DOS_WriteFile (STDOUT,&c, &n);
+                        DOS_ReadFile (STDIN,&c,&n);
+                        do switch (c) {
+                            case 0xD: WriteOut("\n"); goto first_1;
+                            case 0x03: goto next;
+                            case 0x08: WriteOut("\b \b"); goto first_2;
+                        } while (DOS_ReadFile (STDIN,&c,&n));
+                        goto first_2;
+                    }
+                } while (DOS_ReadFile (STDIN,&c,&n));
 			}
+next:
 			// Input has been parsed (pvar[0]=section, [1]=property, [2]=value)
 			// now execute
 			Section* tsec = control->GetSection(pvars[0]);
@@ -1257,7 +1339,8 @@ void CONFIG::Run(void) {
 			std::string inputline = pvars[1] + "=" + value;
 			bool change_success = tsec->HandleInputline(inputline.c_str());
 			if (change_success) {
-				if (!strcasecmp(pvars[0].c_str(), "dosbox")||!strcasecmp(pvars[0].c_str(), "dos")||!strcasecmp(pvars[0].c_str(), "cpu")||!strcasecmp(pvars[0].c_str(), "sdl")||!strcasecmp(pvars[0].c_str(), "ttf")||!strcasecmp(pvars[0].c_str(), "render")) {
+                if (applynew) RebootLanguage("");
+				if (!strcasecmp(pvars[0].c_str(), "dosbox")||!strcasecmp(pvars[0].c_str(), "dos")||!strcasecmp(pvars[0].c_str(), "dosv")||!strcasecmp(pvars[0].c_str(), "cpu")||!strcasecmp(pvars[0].c_str(), "sdl")||!strcasecmp(pvars[0].c_str(), "ttf")||!strcasecmp(pvars[0].c_str(), "render")||!strcasecmp(pvars[0].c_str(), "serial")||!strcasecmp(pvars[0].c_str(), "parallel")) {
 					Section_prop *section = static_cast<Section_prop *>(control->GetSection(pvars[0].c_str()));
 					if (section != NULL) {
 						if (!strcasecmp(pvars[0].c_str(), "dosbox")) {
@@ -1513,6 +1596,10 @@ void CONFIG::Run(void) {
 #if C_PRINTER
                                 if (TTF_using() && printfont) UpdateDefaultPrinterFont();
 #endif
+#endif
+							} else if (!strcasecmp(inputline.substr(0, 7).c_str(), "ptsize=")||!strcasecmp(inputline.substr(0, 8).c_str(), "winperc=")) {
+#if defined(USE_TTF)
+                                ttf_reset();
 #endif
 							} else if (!strcasecmp(inputline.substr(0, 5).c_str(), "lins=")||!strcasecmp(inputline.substr(0, 5).c_str(), "cols=")) {
 #if defined(USE_TTF)
@@ -1531,6 +1618,10 @@ void CONFIG::Run(void) {
                                 ttf_setlines(0, 0);
                                 lastset=0;
 #endif
+							} else if (!strcasecmp(inputline.substr(0, 13).c_str(), "outputswitch=")) {
+                                SetOutputSwitch(section->Get_string("outputswitch"));
+							} else if (!strcasecmp(inputline.substr(0, 7).c_str(), "colors=")) {
+                                if (!strlen(section->Get_string("colors"))) ttf_reset_colors();
 							} else if (!strcasecmp(inputline.substr(0, 3).c_str(), "wp=")) {
 #if defined(USE_TTF)
                                 const char *wpstr=section->Get_string("wp");
@@ -1623,7 +1714,28 @@ void CONFIG::Run(void) {
                                 SetBlinkRate(section);
                                 mainMenu.get_item("ttf_blinkc").check(blinkCursor>-1).refresh_item(mainMenu);
 #endif
+							} else if (!strcasecmp(inputline.substr(0, 4).c_str(), "gbk=")) {
+                                if (gbk != section->Get_bool("gbk")) {
+                                    gbk = !gbk;
+                                    if (enable_dbcs_tables&&dos.tables.dbcs&&(IS_PDOSV||dos.loaded_codepage==936)) mem_writeb(Real2Phys(dos.tables.dbcs)+2,gbk?0x81:0xA1);
+                                    if (dos.loaded_codepage!=950) mainMenu.get_item("ttf_extcharset").check(dos.loaded_codepage==936?gbk:(gbk&&chinasea)).refresh_item(mainMenu);
+#if defined(USE_TTF)
+                                    if (TTF_using()) resetFontSize();
+#endif
+                                }
+							} else if (!strcasecmp(inputline.substr(0, 9).c_str(), "chinasea=")) {
+                                if (chinasea != section->Get_bool("chinasea")) {
+                                    chinasea = !chinasea;
+                                    if (!chinasea) makestdcp950table();
+                                    if (dos.loaded_codepage!=936) mainMenu.get_item("ttf_extcharset").check(dos.loaded_codepage==950?chinasea:(gbk&&chinasea)).refresh_item(mainMenu);
+#if defined(USE_TTF)
+                                    if (TTF_using()) resetFontSize();
+#endif
+                                }
 							}
+						} else if (!strcasecmp(pvars[0].c_str(), "dosv")) {
+                            if (!strcasecmp(inputline.substr(0, 11).c_str(), "fepcontrol=")||!strcasecmp(inputline.substr(0, 7).c_str(), "vtext1=")||!strcasecmp(inputline.substr(0, 7).c_str(), "vtext2="))
+                                DOSV_SetConfig(section);
                         } else if (!strcasecmp(pvars[0].c_str(), "render")) {
                             if (!strcasecmp(inputline.substr(0, 9).c_str(), "glshader=")) {
 #if C_OPENGL
@@ -1633,6 +1745,16 @@ void CONFIG::Run(void) {
 #endif
 							} else if (!strcasecmp(inputline.substr(0, 12).c_str(), "pixelshader="))
                                 GFX_ForceRedrawScreen();
+						} else if (!strcasecmp(pvars[0].c_str(), "serial")) {
+                            if (!strcasecmp(inputline.substr(0, 6).c_str(), "serial") && inputline[7]=='=') {
+                                std::string val = section->Get_string("serial" + std::string(1, inputline[6])), cmd = std::string(1, inputline[6]) + " " + (val.size()?val:"dummy");
+                                runSerial(cmd.c_str());
+                            }
+						} else if (!strcasecmp(pvars[0].c_str(), "parallel")) {
+                            if (!strcasecmp(inputline.substr(0, 8).c_str(), "parallel") && inputline[9]=='=') {
+                                std::string val = section->Get_string("parallel" + std::string(1, inputline[8])), cmd = std::string(1, inputline[8]) + " " + (val.size()?val:"disabled");
+                                runParallel(cmd.c_str());
+                            }
                         }
 					}
 				}
@@ -1735,8 +1857,9 @@ void PROGRAMS_Init() {
 	MSG_Add("PROGRAM_CONFIG_HLP_PROPHLP","Purpose of property \"%s\" (contained in section \"%s\"):\n%s\n\nPossible Values: %s\nDefault value: %s\nCurrent value: %s\n");
 	MSG_Add("PROGRAM_CONFIG_HLP_LINEHLP","Purpose of section \"%s\":\n%s\nCurrent value:\n%s\n");
 	MSG_Add("PROGRAM_CONFIG_HLP_NOCHANGE","This property cannot be changed at runtime.\n");
-	MSG_Add("PROGRAM_CONFIG_HLP_POSINT","positive integer"); 
-	MSG_Add("PROGRAM_CONFIG_HLP_SECTHLP","Section %s contains the following properties:\n");				
+	MSG_Add("PROGRAM_CONFIG_APPLY_RESTART","Do you want to restart now to apply the setting [Y/N]?");
+	MSG_Add("PROGRAM_CONFIG_HLP_POSINT","positive integer");
+	MSG_Add("PROGRAM_CONFIG_HLP_SECTHLP","Section %s contains the following properties:\n");
 	MSG_Add("PROGRAM_CONFIG_HLP_SECTLIST","DOSBox-X configuration contains the following sections:\n\n");
 
 	MSG_Add("PROGRAM_CONFIG_SECURE_ON","Switched to secure mode.\n");
